/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections;
using System.IO;
using System.Xml;
using Gtk;
using Gdk;
using Mono.Data.SqliteClient;
using Mono.Unix;


public class CDIcon {

    public Pixbuf icon;

    /* private */    
    private int height = 32;
    private int width = 32;
    private Gdk.InterpType interp = Gdk.InterpType.Bilinear;
    
        
    public CDIcon (string filename, string serial)
    {
        if (filename!=null) {
            read_icon_from_file (filename);
        } else {
            read_icon_from_string (serial);
        }
    
        /* Make sure icon is heightxwidth */
        scale_icon ();
    }
        
    public CDIcon (Pixbuf pix)
    {
        icon = pix;
        
        /* Make sure icon is heightxwidth */
        scale_icon ();
    }

	public void read_icon_from_file (string name)
	{

		if ( ! System.IO.File.Exists (name) ) {
		
			/* Read the default icon */
			Console.WriteLine (String.Format (Catalog.GetString("Icon '{0}' not found. Using default"),name));
			icon = new Pixbuf( null, "cdcollect_small.png");

		} else {
						
			/* Scale the icon to 48x48 in size */
			try {
				icon = new Pixbuf (name);
            } catch {				
				/* Read the default icon */
				Console.WriteLine (String.Format (Catalog.GetString ("Error scaling icon '{0}'. Using default"), name));
				icon = new Pixbuf( null, "cdcollect_small.png");
			}
	   }	
	}
			
	public void scale_icon ()
	{
	   Pixbuf sicon;
	   
	   if ( (icon.Height==height) & (icon.Width==width) ) {
	       /* Icon is already scaled */
	       sicon = icon;
    	} else {
    		if (icon.Height==icon.Width) {
	  		   sicon = icon.ScaleSimple (width, height, interp);
	  		} else if (icon.Width > icon.Height) {
	  		   int nw = width;
		  	   int nh = (int)(((double)height)/((double)icon.Width)*((double)icon.Height));
		  	   sicon = icon.ScaleSimple (nw, nh, interp);
	  		} else {
	  		   int nh = height;
		  	   int nw = (int)(((double)width)/((double)icon.Width)*((double)icon.Height));
		  	   sicon = icon.ScaleSimple (nw, nh, interp);
	  		}
	  	}
	  	
	  	icon = sicon;  
	}

				
    
    public string get_icon_serial ()
    {
		Pixdata pixdata = new Pixdata ();
		pixdata.FromPixbuf (icon, false);
		
		byte []serial = pixdata.Serialize ();
        
        int kk;
        String s = serial.Length.ToString();
        for (kk=0; kk < serial.Length; kk++) {
            s = s + ":" + ((int)serial[kk]).ToString();
        }
        return (s);    
    }
    
    
    public void read_icon_from_string (string s)
    {
        String []pix = s.Split(':');
        
        byte [] serial = new byte[Convert.ToInt32(pix[0])];
        
        for (int k=1; k<pix.Length; k++) {
            serial[k-1] = (byte)Convert.ToInt32(pix[k]);
        }
        
        try {
            icon = new Pixbuf(serial.Length, serial, true);
        } catch {
            Console.WriteLine (Catalog.GetString ("Couldn't Read icon. Using default")); 
            icon = new Pixbuf (null, "cdcollect_small.png");
        }
    }


    public Pixbuf pixbuf()
    {
        return(icon);
    }

}



