/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections;
using System.IO;
using System.Xml;
using System.Text.RegularExpressions;
using Gtk;
using Gdk;
using Mono.Data.SqliteClient;
using Mono.Unix;

public enum CDItemFind : int {
	Name,         /* find in name */
	Keywords,     /* find in keywords */
	NameKeywords  /* find in name+keywords */
}

public class CDItem {

	private int key;
	public int Key {
		set {
			key = value;
		}
		get {
			return key;
		}
	}

	private string name;
	public string Name {
		set {
			name = value;
		}
		get {
			return name;
		}
	}

	private string keywords;
	public string Keywords {
		set {
			keywords = value;
		}
		get {
			return keywords;
		}
	}

	private DateTime date;
	public DateTime Date {
		set {
			date = value;
		}
		get {
			return date;
		}
	}

	private CDIcon icon;
	public CDIcon Icon {
		set {
			icon = value;
		}
		get {
			return icon;
		}
	}

	private string selected;
	public string Selected {
		set {
			selected = value;
		}
		get {
			return selected;
		}
	}


	public CDItem ( int ke, string n, string k, DateTime d, string iconname, string serial)
	{
		key = ke;
		name = n;
		keywords = k;
		date = d;
		icon = new CDIcon (iconname, serial);
		selected = "";						        				        				        				        				        				        		
	}

	public CDItem ( int ke, string n, string k, DateTime d, Pixbuf ic)
	{
		key = ke;
		name = n;
		keywords = k;
		date = d;
		icon = new CDIcon(ic);
		selected = "";
	}

	public bool Search(string text)
	{
	
		if (text.Length==0) {
	
			return(true);
			
		} else {
		
			if ( name.ToLower().IndexOf(text) >= 0 ) {
				return(true);
			} else if ( keywords.ToLower().IndexOf(text) >= 0 ) {
				return(true);
			}
		
		}
		
		return(false);
	}
	
	public bool SearchFirst(string text)
	{
		if (text.Length==0) {
			return(true);
		} else {
			if ( name.ToLower().IndexOf(text) == 0 ) {
				return(true);
			}
		}
		
		return(false);
	}

	
	public bool Find (string text, bool casesen, CDItemFind opt)
	{
	
	   string ctext, cname, ckeywords;
	   
	   if (casesen) {
	       cname = name;
	       ckeywords = keywords;
	       ctext = text;
	   } else {
	       cname = name.ToLower ();
	       ckeywords = keywords.ToLower ();
	       ctext = text.ToLower ();
        }	       
	   
	   switch (opt) {
    	   case CDItemFind.Name:
    	       return (Regex.IsMatch(cname, ctext));
    	   case CDItemFind.Keywords:
    	       return (Regex.IsMatch (ckeywords, ctext));
    	   case CDItemFind.NameKeywords:
    	       if (Regex.IsMatch (cname, ctext)) {
    	           return (true);
    	       } else {
    	           return (Regex.IsMatch (ckeywords, ctext));
    	       }
	   }
	
	   return (false);
	}

}



