/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
	
using System;
using Gtk;
using Gdk;
using Mono.Unix;


public class CDViewPopup : Gtk.Window
{

	[Glade.Widget] Menu menu_cdview;

	[Glade.Widget] ImageMenuItem menu_edit;
	[Glade.Widget] ImageMenuItem menu_delete;
	
	[Glade.Widget] RadioMenuItem menu_sortdate;
	[Glade.Widget] RadioMenuItem menu_sortname;
	
	/* Events */
	public delegate void CDViewPopupEventHandler(string action);
	public event CDViewPopupEventHandler CDViewPopupEvent;
	
	
	/* Private variables */
	private static Glade.XML gxml;
	
	public CDViewPopup (string sort) : base (IntPtr.Zero)
	{	
		
		/* Get the menu from glade */				
		gxml = new Glade.XML (null, "gui.glade", "menu_cdview", null);
		
		/* Autoconnect events */
		gxml.Autoconnect (this);

        set_sorting(sort);
	}

	public void set_sorting (string sort)
	{
		switch (sort.ToLower()) {
			case "date":
				menu_sortdate.Activate();
				break;
			case "name":
				menu_sortname.Activate();
				break;
		}
	}
	
	public void Popup(int items_sel, uint button, uint time) 
	{
		/* Activate submenus thingy */
		 
		menu_edit.Sensitive = (items_sel==1) ;
		menu_delete.Sensitive = (items_sel>0) ;
		
		/* Show the menu */
		menu_cdview.Popup (null, null, null, button, time);
		
	}   

	
	public void on_menu_sort_activate (object o, EventArgs args)
	{
		if (menu_sortdate.Active) {
			CDViewPopupEvent ("sortdate");
		} else {
			CDViewPopupEvent ("sortname");
		}
	}


	public void on_menu_edit_activate (object o, EventArgs args)
	{
		CDViewPopupEvent ("edit");	
	}

	public void on_menu_delete_activate (object o, EventArgs args)
	{
		CDViewPopupEvent ("delete");	
	}

}
