/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Xml;
using System.Text.RegularExpressions;

using Mono.Unix;

using Gtk;
using Gdk;
using Glade;
using GConf;
using Gnome; 


public class CompPluginWindow : Gtk.Window
{

	/* Glade Widgets */
	[Glade.Widget] public Gtk.Window win_compplugin;

	[Glade.Widget] public Gtk.Button btn_ok;
	[Glade.Widget] public Gtk.Button btn_cancel;
		
	[Glade.Widget] public Gtk.Entry entry_mime;
	[Glade.Widget] public Gtk.Entry entry_program;
	[Glade.Widget] public Gtk.Entry entry_args;	
		
	/* Public variables */
        
	/* Private variables */
	private static Glade.XML gxml;
	private string oldmime;
	private FileSelection fileselec;	
		
	/* Event to trigger when adding a Item */
	public delegate void AddEventHandler(CompPluginItem item, string oldmime);
	public event AddEventHandler AddEvent;
    
    
    /* Methods */
	public CompPluginWindow (CompPluginItem c, Gtk.Window parent) : base (IntPtr.Zero) 
	{
		/* Show Main Dialog using glade */
 		gxml = new Glade.XML (null, "gui.glade", "win_compplugin", null);
			
		/* Connect functions to events */
		gxml.Autoconnect (this);

		win_compplugin.TransientFor = parent;
				
		/* Create file selection */
		fileselec = new FileSelection(Catalog.GetString ("Program Selection"));	
		fileselec.OkButton.Clicked += file_ok_event;
        fileselec.CancelButton.Clicked +=new EventHandler (file_cancel_event);

		/* Prepare gui */
		prepare_gui ();
		
		/* Set CompPlugin properties */
		oldmime = c.Mime;
		
		entry_mime.Text = c.Mime;
		entry_program.Text = c.Program;
		entry_args.Text = c.Args;
		
		/* Show all widgets */
		win_compplugin.Modal = true;
		win_compplugin.ShowAll ();
           				
		/* Select focus */           											    																		    											
        btn_cancel.GrabFocus();
        
	}

    /* Methods */        
	public CompPluginWindow (Gtk.Window parent) : base (IntPtr.Zero) 
	{
		/* Show Main Dialog using glade */
 		gxml = new Glade.XML (null, "gui.glade", "win_compplugin", null);
			
		/* Connect functions to events */
		gxml.Autoconnect (this);
										
		win_compplugin.TransientFor = parent;

		/* Create file selection */
		fileselec = new FileSelection(Catalog.GetString ("Program Selection"));	
		fileselec.OkButton.Clicked += file_ok_event;
        fileselec.CancelButton.Clicked +=new EventHandler (file_cancel_event);

		/* Prepare gui */
		prepare_gui ();

		/* Set CompPlugin properties */
		oldmime = "";
		
		/* Show all widgets */
		win_compplugin.Modal = true;
		win_compplugin.ShowAll ();
           				
		/* Select focus */           											    																		    											
        entry_mime.GrabFocus();
        
	}

	private void prepare_gui()
	{
		/* Title */
		win_compplugin.Icon = new Pixbuf(null, "cdcollect_small.png");
		win_compplugin.Title = Catalog.GetString ("CDCollect: Compression Plugin");

		/* Read GUI gconf properties */
		int win_height, win_width;
      		 
		/* Window size */
		win_height = (int) CDCollect.conf.getkey ("gui/compplugin_height", 100);
		win_width  = (int) CDCollect.conf.getkey ("gui/compplugin_width", 200);
		win_compplugin.Resize( win_width, win_height);
	}

	public void on_win_size_allocate (object o, Gtk.SizeAllocatedArgs args)
	{
		/* Save window position */
		int h,w;
        	
		win_compplugin.GetSize(out w, out h);
        	
       	CDCollect.conf.setkey ("gui/compplugin_height", h);
		CDCollect.conf.setkey ("gui/compplugin_width", w);
	}
	
	private bool check()
	{
		/* Check for correct input mime */
		if (entry_mime.Text=="") {
			new ErrorDialog (Catalog.GetString ("Unvalid mime, please enter a valid mime type."), win_compplugin);
			return(false);
		}
		/* Check for correct program name */
		if (entry_program.Text=="") {
			new ErrorDialog (Catalog.GetString ("Unvalid program name, please enter a valid external command."), win_compplugin);
			return(false);
		}
		/* Check for correct program name */
		if ( ! System.IO.File.Exists(entry_program.Text) ) {
			new ErrorDialog (Catalog.GetString ("Unvalid program name, please enter a valid external command."), win_compplugin);
			return(false);
		}
		return(true);
	}

	public void on_ok_clicked (object o, EventArgs args)
	{
		CompPluginItem c;
		
		if ( check() ) {

			/* Import a CD into the database */
			win_compplugin.Hide ();
			c = new CompPluginItem (entry_mime.Text, entry_program.Text, entry_args.Text); 	
			AddEvent (c, oldmime);
			win_compplugin.Destroy ();
		}
	}

	public void on_cancel_clicked (object o, EventArgs args)
	{
		win_compplugin.Destroy();
	}

    public void on_delete_event (object o, DeleteEventArgs args) 
	{
		win_compplugin.Destroy();
	}

	public void on_btn_browse_clicked (object o, EventArgs args)
	{
		fileselec.Filename = entry_program.Text;
		fileselec.Modal = true;
		fileselec.Show ();
	}

	private void file_ok_event( object o, EventArgs args )
	{
		entry_program.Text = fileselec.Filename;
		fileselec.Hide ();
	}

	private void file_cancel_event( object o, EventArgs args )
	{
		fileselec.Hide ();
	}

}

