/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Data;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Xml;
using System.Text;
using System.Threading;
using System.Text.RegularExpressions;

using Mono.Data.SqliteClient;
using Mono.Unix;


using Gtk;
using Gdk;


/*

    import: read from args or external and put things in DB and hashtables
    delete: delete contents from DB and hashtables
    update: update contents in DB and hashtables
    
    read : Read contents from DB to args
    write: Write contents from args to DB
    
*/

public class CDDatabaseSql
{
	/* Public variables */
	public string dbfilename;
	public static IDbConnection sqlite_connection;
	
	/* Private variables */
	protected static string SqlString (string s)
	{
		return (s.Replace ("'","''"));
	}

	private ConvertDBProgressWindow win_convert;
	private static System.Threading.Thread thread = null;
	private bool thread_finished;
		
	private string thumbsdir;

	/* CONSTRUCTOR */
	public CDDatabaseSql(string dbname,string thumbsdir)
	{

		this.dbfilename = dbname;
		this.thumbsdir = thumbsdir;
				
		/* Open Database */
		if ( ! System.IO.File.Exists(dbfilename) ) {
            
			/* Create Database */
			sqlite_connection = new SqliteConnection ();
			sqlite_connection.ConnectionString = "version=3,URI=file:" + dbfilename + "";
			//sqlite_connection.ConnectionString = "URI=file:" + dbfilename + "";
	
			sqlite_connection.Open ();
			  
			/* Create Table for CD items */	
			Console.WriteLine (Catalog.GetString ("Creating cds SQL table."));			
			create_cds_table ();
            	
			/* Create Table for CD files */	
			Console.WriteLine (Catalog.GetString ("Creating files SQL table"));			
			create_files_table ();

			/* Create Table for infoplugins items */	
			Console.WriteLine (Catalog.GetString ("Creating infoplugins SQL table"));
			create_infos_table ();
			create_infos_initial ();
            
			/* Create Table for compression plugins items */
			Console.WriteLine (Catalog.GetString ("Creating compplugins SQL table"));
			create_comps_table ();
			create_comps_initial ();

			/* Create Table for version info */
			Console.WriteLine( Catalog.GetString ("Creating cdcollect_version SQL table"));
			create_version_table ();
			create_version_initial ();				

		} else {
		
			sqlite_connection = new SqliteConnection ();
			sqlite_connection.ConnectionString = "version=3,URI=file:" + dbfilename + "";	
			//sqlite_connection.ConnectionString = "URI=file:" + dbfilename + "";	
			sqlite_connection.Open ();
						
			/* Check for current version (if not convert it) */
			check_and_convert_db ();

		}

    }
    
    public int get_version ()
    {
    	IDbCommand command;
    	IDataReader reader;
		int dbver = 0;
		
        command = sqlite_connection.CreateCommand ();

		command.CommandText = "SELECT ver FROM cdcollect_version";

		try {
			reader = command.ExecuteReader ();
			while (reader.Read ()) {
				dbver = Convert.ToInt32(reader["ver"]);
			}
			reader.Close ();
		} catch {
			/* No version table, add one */
			create_version_table ();
			create_version_initial ();
			dbver = get_version ();
		}
  		command.Dispose ();
  		
		return (dbver);        	
    }


  
	private void check_and_convert_db ()
	{
  
		int dbver = get_version ();

		if ( dbver != 4 ) {
		
			OKCancelDialog dialog = new OKCancelDialog(
				null,
				Catalog.GetString (	"The CDCollect database format has changed.\n" + 
									"In order to continue CDCollect must convert the database format\n" + 
									"(no data will be lost).\n\n" + 
									"The process may take some time,\n" + 
									"depending on database size and number of files."), 
				null);
				
			if (dialog.GetAnswer () == true) {
					
				/* Convert db */
				Trace.WriteLine ( String.Format (Catalog.GetString("Converting DB from version {0} to version 1."),dbver));
				
				/* Create Conversion Window */
				win_convert = new ConvertDBProgressWindow(null);

				/* Do everything in its own thread */
				thread = new System.Threading.Thread (new System.Threading.ThreadStart(convert_thread));
				thread_finished = false;
				thread.Start ();
				
				/* Wait for the thread to finish */
				while (thread_finished==false) {
					//System.Threading.Thread.Sleep (2000);
					Gtk.Application.RunIteration (false);
				}
	
				/* Destroy progress window */
				win_convert.Finish ();

				/* Inform the user */
	            OKDialog ok = new OKDialog (null, Catalog.GetString("Conversion finished successfully."), null);
				ok.GetAnswer (); 
															
			} else {
				
				/* Stop the program */
				sqlite_connection.Close ();
				System.Environment.Exit (0);
			
			}

		}

	}

    private void convert_thread ()
    {
    	IDbCommand command;
    	IDataReader reader;
		
        command = sqlite_connection.CreateCommand ();

		int dbver = get_version ();
    	switch (dbver) {
    		case 0:

				/* Create Conversion Window */
				win_convert.action = Catalog.GetString("Creating new files SQL table.");
				ConvertDBProgressWindow.actionnotify.WakeupMain ();
				create_files_table ();

				/* Move files to new table */
				win_convert.action = Catalog.GetString("Reading CDs in DB.");
				ConvertDBProgressWindow.actionnotify.WakeupMain ();
				Hashtable cds = read_cds ();
				win_convert.bar_total = cds.Count;


				int ncd = 1;
				foreach (CDItem cd in cds.Values) {
				
					win_convert.bar_steps = ncd-1;
					win_convert.bar_text = String.Format ("CD {0} of {1}",ncd,cds.Count);
					ConvertDBProgressWindow.stepnotify.WakeupMain ();
					
					win_convert.action = Catalog.GetString("Reading files in DB.");
					ConvertDBProgressWindow.actionnotify.WakeupMain ();

					/* Read all files in the cd */
					Hashtable files = new Hashtable();
					Console.WriteLine ( String.Format(Catalog.GetString("Converting files in cd '{0}' ({1}/{2})"),
						cd.Name, ncd, cds.Count));
			

					command.CommandText = String.Format (
						"SELECT path, name, info, size, mime, type, packed FROM files " + 
						"WHERE cd_id = {0}                                ", 
						cd.Key
					);
						
					reader = command.ExecuteReader ();

					while ( reader.Read() ) {
		
						/* Create a file Item */
						FileItem file = new FileItem (	cd.Key,
														reader["path"].ToString(), 
							 	 						reader["name"].ToString(),
							  							reader["info"].ToString(), 
								  						Convert.ToInt32(reader["size"]),
								  						reader["mime"].ToString(),
								  						"",
								  						(FileItemType)Convert.ToInt32(reader["type"]),
							  							(Convert.ToInt32(reader["packed"])==0) ? false : true
 					 								);
			
						/* Add it to the hash table */
						files.Add( file.Path, file);
				
					}
		
					reader.Close ();
					command.Dispose ();
					

					win_convert.action = Catalog.GetString("Updating Location field of files.");
					ConvertDBProgressWindow.actionnotify.WakeupMain ();

					/* Update the location for each file */
					foreach (FileItem f in files.Values) {

						/* Get the location by traversing the path */
						string []nodes = Regex.Split( f.Path, ":");
						string lpath = "";
						string loca = "/";
		
						for (int i=0; i<(nodes.Length-1); i++) {
			
							/* Create path for parent nodes */
							lpath = lpath + nodes[i];
			
							/* Add location */
							if (files.Contains(lpath)) {
								loca = loca + ((FileItem)files[lpath]).Name;
								if (i<(nodes.Length-2)) loca = loca + "/";
							}
		
							lpath = lpath + ":";
			 
						}
		
						/* Write location into file */
						f.Location = loca;
					}
					
					/* Write files back to the DB */
			        foreach (FileItem f in files.Values) {
						string name;						
						if (f.Name.Length>30) {
							name = "'" + f.Name.Substring(0,19) + "..." + f.Name.Substring(f.Name.Length-8,8) + "'";
						} else {
							name = "'" + f.Name + "'";
						}
						win_convert.action = String.Format (Catalog.GetString("Converting file {0}."),name);
						ConvertDBProgressWindow.actionnotify.WakeupMain ();
			            write_file (cd.Key, f.Path, f.Name, f.Info, f.Size, f.Mime, f.Location, f.Type, f.Packed);
			        }
					//write_files (cd.Key, files);
				
					ncd++;
						
				} /* foreach CD */
				
				
				/* Create version table and add the version */
				win_convert.action = Catalog.GetString("Deleting old files SQL table.");
				ConvertDBProgressWindow.actionnotify.WakeupMain ();
	            sql_command ("DROP TABLE files");

				/* Create version table and fill it */
				win_convert.action = Catalog.GetString("Creating new version table information.");
				ConvertDBProgressWindow.actionnotify.WakeupMain ();
				create_version_table ();
				create_version_initial ();
	            
	            Console.WriteLine (Catalog.GetString ("Conversion finished."));
				win_convert.action = Catalog.GetString ("Conversion finished.");
				ConvertDBProgressWindow.actionnotify.WakeupMain ();


	            thread_finished = true;
			
				break;
				
    	}
        	
    }

    public Hashtable read_infos ()
    {
        Hashtable infoplugins = new Hashtable ();
        IDbCommand command;
  		IDataReader reader;
      
		/* Read Info Plugins */
		command = sqlite_connection.CreateCommand ();

		command.CommandText = "SELECT mime, program, arguments, regex_input, regex_pattern, regex_case FROM infoplugins";
		reader = command.ExecuteReader ();

		while (reader.Read ()) {

			/* Read plugin  */
			InfoPluginItem plugin = new InfoPluginItem (
				reader["mime"].ToString(), 
				reader["program"].ToString(),
				reader["arguments"].ToString(),
				reader["regex_input"].ToString(), 
				reader["regex_pattern"].ToString(), 
				Convert.ToInt32(reader["regex_case"])==0 ? false : true
	 		);
				
			/* Add it to the hash table */
			infoplugins.Add(plugin.Mime, plugin);

		}
		reader.Close ();
		command.Dispose ();        

        return(infoplugins);
    }
    
    public Hashtable read_comps ()
    {
        Hashtable compplugins = new Hashtable ();
		IDbCommand command;
    	IDataReader reader;

		/* Read Comp Plugins */
		command = sqlite_connection.CreateCommand ();
		command.Connection = sqlite_connection;

		command.CommandText = "SELECT mime, program, arguments FROM compplugins";
		reader = command.ExecuteReader ();

		while (reader.Read ()) {

			/* Read plugin  */
			CompPluginItem c = new CompPluginItem (
				reader["mime"].ToString(), 
				reader["program"].ToString(),
				reader["arguments"].ToString()
	 		);
				
			/* Add it to the hash table */
			compplugins.Add(c.Mime, c);

		}
		reader.Close ();
		command.Dispose ();        
        
        return (compplugins);
	}
	


    public int write_cd (string name, string keywords, DateTime date, string iconserial)
    {
		int key;    
    
    	key = sql_command_id (
    			String.Format(
				    "INSERT INTO cds (name, keywords, year, month, day, icon) " +
			 	   "            VALUES ('{0}', '{1}', {2}, {3}, {4}, '{5}')   ",
	 			   SqlString(name), SqlString(keywords), date.Year, date.Month, date.Day, SqlString(iconserial)
	 		    )
	 	     );
	 	     
    	// Create directory to store thumbnails
		System.IO.Directory.CreateDirectory (thumbsdir + System.IO.Path.DirectorySeparatorChar + key.ToString());
		
		return (key);
		
	}		

		
	public void update_cd (CDItem cd)
	{			
		/* Update CD in SQL table */
		sql_command ( 
			String.Format (
				"UPDATE cds SET       " +
				"   name = '{0}',     " +
				"   keywords = '{1}', " + 
				"   year = {2},       " +
				"   month = {3},      " +
				"   day = {4},        " +
				"   icon = '{5}'      " +
				"WHERE id = {6}       ",
				SqlString (cd.Name),
				SqlString (cd.Keywords),
				cd.Date.Year,
				cd.Date.Month,
				cd.Date.Day,
				cd.Icon.get_icon_serial(),
				cd.Key
			)
		);
			
	}

    public void write_files (int cd_id, Hashtable files)
    {
        foreach (FileItem f in files.Values) {
            write_file (cd_id, f.Path, f.Name, f.Info, f.Size, f.Mime, f.Location, f.Type, f.Packed);
        }
    }

    public void write_file ( int cd_id, string path, string name, string info, int size, string mime, 
                             string location, FileItemType type, bool packed)
    {
       
       	sql_command (
	   	   String.Format( 
	   	       "INSERT INTO files_v1 (cd_id, path, name, info, size, mime, location, type, packed) " +
			   "            VALUES ({0}, '{1}', '{2}', '{3}', {4}, '{5}', '{6}', {7}, {8})   ",
			   cd_id, 
			   SqlString(path), 
			   SqlString(name),
			   SqlString(info), 
			   size, SqlString(mime),
			   SqlString(location), 
			   (type==FileItemType.Directory) ? 0 : 1, 
			   (packed) ? 1 : 0)
		);

		/*
		// FIXME: Escape strings ?
		
		Console.WriteLine ("TRYING TO INSERT DATA TO DB");
		string sqlstring =  "INSERT INTO files_v1 (cd_id, path, name, info, size, mime, location, type, packed) " +
			   "            VALUES (:cd_id_int, :path_string, :name_string, :info_string, :size_int, :mime_string, :location_string, :type_int, :packed_int)";



		SqliteCommand command = new SqliteCommand ();
		command.Connection = (SqliteConnection)sqlite_connection;
		command.CommandText = sqlstring;
		

		command.Parameters.Add (":cd_id_int",   cd_id);
		command.Parameters.Add (":path_string", path);
		command.Parameters.Add (":name_string", name);
		command.Parameters.Add (":info_string", info);
		command.Parameters.Add (":size_int",    size);
		command.Parameters.Add (":mime_string", mime);
		command.Parameters.Add (":location_string", location);
		command.Parameters.Add (":type_int", (type==FileItemType.Directory) ? 0 : 1);
		command.Parameters.Add (":packed_int", (packed) ? 1 : 0);


	
		Console.WriteLine ("DONE string: " + sqlstring);
		Console.WriteLine ("DONE string command: " + command.CommandText);

		command.ExecuteNonQuery ();
		Console.WriteLine ("TRYING TO Dispose");
		
		command.Dispose ();

		Console.WriteLine ("TRYING TO INSERT DATA TO DB: DONE!!!!!!!!!");
		*/
	}

	public void write_info (InfoPluginItem p) 
	{		
		/* Add plugin to DB */
		sql_command (
			String.Format (
				"INSERT INTO infoplugins (mime, program, arguments, regex_input, regex_pattern, regex_case) " +
			 	"            VALUES ('{0}', '{1}', '{2}', '{3}', '{4}', {5})   ",
	 			SqlString(p.Mime), SqlString(p.Program), SqlString(p.Args), 
	 			SqlString(p.RegexInput), SqlString(p.RegexPattern), p.RegexCase ? 1 : 0  
	 		)
	 	);		
	}
	
	public void write_comp (CompPluginItem c) 
	{
		/* Add plugin to DB */
		sql_command (
			String.Format (
				"INSERT INTO compplugins (mime, program, arguments) " +
			 	"            VALUES ('{0}','{1}','{2}')   ",
	 			SqlString(c.Mime), SqlString(c.Program), SqlString(c.Args)  
	 		)
	 	);		
	}

	public void delete_info (string mime)
	{
		/* Remove it from the DB */
		sql_command ( String.Format ("DELETE FROM infoplugins WHERE mime = '{0}'", SqlString(mime)) );																		
	}
	
	public void delete_comp (string mime)
	{
		/* Remove it from the DB */
		sql_command ( String.Format ("DELETE FROM compplugins WHERE mime = '{0}'", SqlString(mime)) );																		
	}

	private void create_cds_table () 
	{
		sql_command (
			"CREATE TABLE cds (                                " +
			"       id            INTEGER PRIMARY KEY NOT NULL," +
			"       name          TEXT,                        " +
			"       keywords      TEXT,		                   " +
			"       year          INTEGER,		               " +
			"       month         INTEGER,	                   " +
			"       day           INTEGER,	                   " +
			"       icon          TEXT                         " +
			")");
	}

	private void create_files_table ()
	{
		sql_command (
			"CREATE TABLE files_v1 (       " +
			"       cd_id         INTEGER, " +
			"       path          TEXT,    " +
			"       name          TEXT,	   " +
			"       info          TEXT,    " +
			"       size          INTEGER, " +
			"       mime          TEXT,    " +
			"       location      TEXT,    " +
			"       type          INTEGER, " +
			"       packed        INTEGER  " +
			")");
	}

	private void create_infos_table () 
	{
		sql_command (
			"CREATE TABLE infoplugins (        " +
			"       mime          TEXT UNIQUE, " +
			"       program       TEXT,	       " +
			"       arguments     TEXT,	       " +
			"       regex_input   TEXT,	       " +
			"       regex_pattern TEXT,	       " +
			"       regex_case    INTEGER      " +
			")");
	}
	private void create_infos_initial ()
	{
		
	}

	private void create_comps_table ()
	{
		sql_command (
			"CREATE TABLE compplugins (        " +
			"       mime          TEXT UNIQUE, " +
			"       program       TEXT,	       " +
			"       arguments     TEXT	       " +
			")");
	}

    public void create_comps_initial()
    {
        string []p = {
            "application/x-arj", 					"/usr/bin/arj", "e {0} -w {1}", 
            "application/x-bzip-compressed-tar",	"/bin/tar",		"xjf {0} -C {1}",
            "application/x-compressed-tar",			"/bin/tar", 	"xzf {0} -C {1} ",
            "application/x-lha", 					"/usr/bin/lha",	"e {0} w={1}",
            "application/x-rar", 					"/usr/bin/rar",	"e {0} {1}/",
            "application/x-tar",					"/bin/tar",		"xf {0} -C {1}", 
            "application/zip",						"/usr/bin/unzip",	"{0} -d {1}"
        };
        
        for (int i=0; i<(p.Length)/3; i++) {
            sql_command_id (
                String.Format ( "INSERT INTO compplugins (mime, program, arguments) VALUES ('{0}','{1}','{2}')",
				    SqlString(p[3*i]), SqlString(p[3*i+1]), SqlString(p[3*i+2]) )
		    );
        }
    }

	private void create_version_table ()
	{
		sql_command (
			"CREATE TABLE cdcollect_version (        " +
			"       ver           INTEGER  " +
			")");
	}

	private void create_version_initial ()
	{
		sql_command ( "INSERT INTO cdcollect_version (ver) VALUES (4)" );	
	}



    public void check_contents ()
    {
        IDbCommand command;
        IDataReader reader;
        int Ncds = 0;
        int Nfiles = 0;
      
        command = sqlite_connection.CreateCommand ();
		command.CommandText = "SELECT id FROM cds";
   		reader = command.ExecuteReader ();
		while (reader.Read ()) {
            Ncds++;
		}
		reader.Close ();
		command.Dispose ();
		
        command = sqlite_connection.CreateCommand ();
		command.CommandText = "SELECT path FROM files_v1";
   		reader = command.ExecuteReader ();
		while (reader.Read ()) {
            Nfiles++;
		}
		reader.Close ();
		command.Dispose ();
		
		Console.WriteLine (String.Format (Catalog.GetString ("There are {0} CDs and {1} files in the CDCollect DB"),Ncds, Nfiles));
	}

    public int get_total_files ()
    {
        IDbCommand command;
        IDataReader reader;
        int Nfiles = 0;
      		
        command = sqlite_connection.CreateCommand ();
		command.CommandText = "SELECT path FROM files_v1";
   		reader = command.ExecuteReader ();

		while (reader.Read ()) {
            Nfiles++;
		}

		reader.Close ();
		command.Dispose ();

		return (Nfiles);		
	}



	public ArrayList find_files (	bool doname, string nametype, string namefind, bool namecase,
									bool dosize, string sizetype, int sizefind)
	{
		IDbCommand command;
		IDataReader reader;
		ArrayList files = new ArrayList();
					
		command = sqlite_connection.CreateCommand ();

		if ( (doname) && (!dosize) ) {
		
			/* Search only on name */
			if (namecase) {				
				command.CommandText = String.Format (
						"SELECT path, name, info, size, mime, location, type, packed, cd_id FROM files_v1 " + 
							"WHERE {0} GLOB '*{1}*' OR path GLOB '*{1}*' AND type=1", 
						nametype, SqlString(namefind) 
				);
			} else {
				command.CommandText = String.Format (
						"SELECT path, name, info, size, mime, location, type, packed, cd_id FROM files_v1 " + 
						"WHERE {0} LIKE '%{1}%' OR path LIKE '%{1}%' AND type=1", 
						nametype, SqlString(namefind)	
				);																				
			}
		}
		
		if ( (doname) && (dosize) ) {
		
			/* Search on name AND size*/
			if (namecase) {				
				command.CommandText = String.Format (
						"SELECT path, name, info, size, mime, location, type, packed, cd_id FROM files_v1 " + 
						"WHERE ({0} GLOB '*{1}*' OR path GLOB '*{1}*') AND type=1 AND size {2} {3}",
						nametype, SqlString(namefind), sizetype, sizefind
				);
			} else {
				command.CommandText = String.Format (
						"SELECT path, name, info, size, mime, location, type, packed, cd_id FROM files_v1 " + 
						"WHERE ({0} LIKE '%{1}%' OR path LIKE '%{1}%') AND type=1 AND size {2} {3}",
						nametype, SqlString(namefind), sizetype, sizefind
				);																				
			}
		}

		if ( (!doname) && (dosize) ) {
			command.CommandText = String.Format (
					"SELECT path, name, info, size, mime, location, type, packed, cd_id FROM files_v1 " + 
					"WHERE size {0} {1}", 
					sizetype, sizefind 
			);
		}		

		if ( (!doname) && (!dosize) ) {
			Console.WriteLine("doname or dosize must be TRUE!!");
		}		
			
		Trace.WriteLine ("Executing SQL query = \"" + command.CommandText);

		reader = command.ExecuteReader ();

		while ( reader.Read() ) {
		
			/* Create a file Item */
			FileItem file = new FileItem (
								Convert.ToInt32(reader["cd_id"]),
								reader["path"].ToString(), 
						 	 	reader["name"].ToString(),
						  		reader["info"].ToString(), 
							  	Convert.ToInt32(reader["size"]),
							  	reader["mime"].ToString(),
							  	reader["location"].ToString(),
							  	(FileItemType)Convert.ToInt32(reader["type"]),
						  		(Convert.ToInt32(reader["packed"])==0) ? false : true
	 					 	);
			
			/* Add it to the list */
			files.Add(file);
				
		}
		
		reader.Close ();
		command.Dispose ();
			
		return(files);
	}

	public Hashtable read_files (int cd_id)
	{
	
		Hashtable files = new Hashtable();		

		IDbCommand command = sqlite_connection.CreateCommand ();	
		command.CommandText = String.Format (
				"SELECT path, name, info, size, mime, location, type, packed FROM files_v1 WHERE cd_id = {0}", 
				cd_id
		);
						
		IDataReader reader = command.ExecuteReader ();

		while ( reader.Read () ) {

			/*
			string info;
			// Read info with a try as sometimes is broken :-?
			try {
				info = (string)reader["info"];
			} catch (Exception e) {
				Console.WriteLine ("Something happended while reading the information of file '" +
					+ (string)reader["location"] + "/" + (string)reader["name"] + "'");
				info = "DOES NOT WORK";
			}
			*/			

			/* Create a file Item */
			FileItem file = new FileItem (
								cd_id,
								reader["path"].ToString(), //(string)reader["path"], 
						 	 	reader["name"].ToString(), //(string)reader["name"],
						  		reader["info"].ToString(), //(string)reader["info"], 
							  	Convert.ToInt32(reader["size"]),
							  	reader["mime"].ToString(), //(string)reader["mime"],
							  	reader["location"].ToString(), //(string)reader["location"],
							  	(FileItemType)Convert.ToInt32(reader["type"]),
						  		(Convert.ToInt32(reader["packed"])==0) ? false : true
	 					 	);
			
			/* Add it to the hash table */
			files.Add (file.Path,file);
				
		}
		
		reader.Close ();
		command.Dispose ();

		return(files);
	}

    public Hashtable read_cds ()
    {        
        Hashtable cds = new Hashtable ();
        IDbCommand command;
   		IDataReader reader;

		/* Read CD Database */
		command = sqlite_connection.CreateCommand ();

		command.CommandText = "SELECT id, name, keywords, year, month, day, icon FROM cds";
		reader = command.ExecuteReader ();

		while (reader.Read ()) {

			int key;
			string name, keywords, iconserial;
			DateTime date;

			/* Read CD properties */
			key = Convert.ToInt32 (reader["id"]);
			name = reader["name"].ToString ();
			keywords = reader["keywords"].ToString ();
			date = new DateTime( Convert.ToInt32 (reader["year"]), Convert.ToInt32 (reader["month"]),
				Convert.ToInt32 (reader["day"]) );
			iconserial = reader["icon"].ToString();				

			/* Create CDItem */
			CDItem cd = new CDItem (key, name, keywords, date, null, iconserial);

			/* Add it to the hashtable */
			cds.Add (cd.Key, cd);
			
		}
		reader.Close ();
		command.Dispose ();
		
		return(cds);
	}

    
	public Hashtable read_cds (ArrayList files)
	{
		Hashtable cds = new Hashtable();
		FileItem f;
	
		for (int i=0; i<files.Count; i++) {
		
			f = (FileItem) files[i];
			
			/* Get the cd if not in the hashtable */			
			if ( cds.Contains (f.CD_Key) ) {
				// Nothing, already there
			} else {
				/* Read cd from Database */
				CDItem cd = read_cds (f.CD_Key);
				cds.Add (cd.Key, cd);
			}
			
		}
		
		return(cds);
	}
	
	public CDItem read_cds (int key)
	{
	
		IDbCommand command = sqlite_connection.CreateCommand ();

		command.CommandText = String.Format (
				"SELECT name, keywords, year, month, day, icon FROM cds " + 
				"WHERE id = {0}                                         ", 
				key.ToString()
		);


		IDataReader reader = command.ExecuteReader ();

		string name = "";
		string keywords = "";
		string iconserial = "";
		DateTime date = new DateTime();;

		while ( reader.Read() ) {

			/* Read CD properties */
			name = reader["name"].ToString ();
			keywords = reader["keywords"].ToString ();
			date = new DateTime( Convert.ToInt32 (reader["year"]), Convert.ToInt32 (reader["month"]),
				Convert.ToInt32 (reader["day"]) );
			iconserial = reader["icon"].ToString();				

		}
		reader.Close ();
		command.Dispose ();
						
		/* Create CDItem */
		CDItem cd = new CDItem (key, name, keywords, date, null, iconserial);
		
		return(cd);
	}
/*
	public void delete_cds (Hashtable cds_sel)
	{
		foreach (CDItem cd in cds_sel.Values) {
			delete_cds (cd.Key);
		}
	}
*/											
	public void delete_cds (int key)
	{
		sql_command ( String.Format ("DELETE FROM cds WHERE id = {0}", key) );
	}
											
	public void delete_files (int key)
	{
		sql_command ( String.Format ("DELETE FROM files_v1 WHERE cd_id = {0}", key) );
	}

	private int sql_command_id (string comm)
	{	
		IDbCommand command = sqlite_connection.CreateCommand ();
		command.CommandText = comm;
		command.ExecuteScalar ();
		command.Dispose ();
		return (((SqliteConnection)sqlite_connection).LastInsertRowId);
	}
	
	private void sql_command (string comm)
	{	
		IDbCommand command = sqlite_connection.CreateCommand ();
		command.CommandText = comm;
		command.ExecuteNonQuery ();
		command.Dispose ();
	}

	public Pixbuf read_thumbnail (int CDKey, string thumbkey)
	{
		string thumbfile = thumbsdir + CDKey.ToString()
			+ System.IO.Path.DirectorySeparatorChar + thumbkey + ".png";
				
    	if (System.IO.File.Exists (thumbfile)) {
    		return (new Pixbuf (thumbfile));
    	} else {
       		return (null);
    	}		
	}
	
	public void write_thumbnail (Pixbuf thumb, int CDKey, string thumbkey)
	{
		string thumbfile = thumbsdir + CDKey.ToString()
			+ System.IO.Path.DirectorySeparatorChar + thumbkey + ".png";

		thumb.Save (thumbfile,"png");		
	}
	
	public void delete_thumbnails (int CDKey)
	{
		/* Delete all thumbnails corresponding to CDkey */
		Trace.WriteLine ("Deleting thumbsnails for CD = " + CDKey.ToString());
		try {
			System.IO.Directory.Delete (thumbsdir + CDKey.ToString(),true);
		} catch (System.IO.DirectoryNotFoundException e) {
			// Directory not found, do not delete anything
		}
	}
	
	public void delete_thumbnail (int CDKey,string thumbkey)
	{
		string thumbfile = thumbsdir + CDKey.ToString()
			+ System.IO.Path.DirectorySeparatorChar + thumbkey + ".png";
		try {
			System.IO.File.Delete (thumbfile);
		} catch {
			// Thumbnail not found
		}
	}

	public void update_file (FileItem f)
	{			
		/* Update FILE in SQL table */
		sql_command ( 
			String.Format (
				"UPDATE files_v1 SET                  " +
				"   info = '{0}'                      " +
				"WHERE cd_id = {1} AND path = '{2}'   ",
				SqlString (f.Info), f.CD_Key, f.Path
			)
		);			
	}

}

