/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Xml;

using Mono.Unix;

using Gtk;
using Gdk;
using Glade;
using GConf;
using Gnome; 


public class EditCDWindow : Gtk.Window
{

	/* Glade Widgets */
	[Glade.Widget] public Gtk.Window win_importcd;
	[Glade.Widget] public Gtk.Button btn_ok;
	[Glade.Widget] public Gtk.Button btn_cancel;
	[Glade.Widget] public Gtk.Entry  entry_name;
	[Glade.Widget] public Gtk.Entry  entry_keywords;
	
	[Glade.Widget] public Gnome.DateEdit  dateedit;

	[Glade.Widget] public Gtk.Button btn_icon;
	[Glade.Widget] public Gtk.Entry  entry_icon;
	[Glade.Widget] public Gtk.Image  img_icon;

	/* Public variables */
        
	/* Private variables */
	private static Glade.XML gxml;
	//private DateTime date;
	//private string mount;
	private CDItem cd;

	private FileSelection fileselec;
	
	/* Event to trigger when adding a CD */
	public delegate void EditCDEventHandler(CDItem cd);
	public event EditCDEventHandler EditCDEvent;
    
    /* Methods */        
	public EditCDWindow (CDItem cdin, Gtk.Window parent) : base (IntPtr.Zero) 
	{
		/* Show Main Dialog using glade */
 		gxml = new Glade.XML (null, "gui.glade", "win_importcd", null);
			
		/* Connect functions to events */
		gxml.Autoconnect (this);
								
		win_importcd.TransientFor = parent;

		cd = cdin;
		
		/* Set CD properties */
		entry_name.Text = cd.Name;
		entry_keywords.Text = cd.Keywords;

       	// Set DateEdit time
		dateedit.Time = cd.Date;
		
		/* Set the icon */
		img_icon.Pixbuf = cd.Icon.pixbuf();

		/* Create file selection */
		fileselec = new FileSelection (Catalog.GetString ("Icon Selection"));	
		fileselec.OkButton.Clicked += file_ok_event;
        fileselec.CancelButton.Clicked +=new EventHandler (file_cancel_event);

		/* Show all widgets */
		win_importcd.Modal = true;
		win_importcd.ShowAll ();
           				
		/* Select focus */
		btn_cancel.GrabFocus();
        
	}

	public void prepare_gui()
	{
		/* Title */
		win_importcd.Icon = new Pixbuf(null, "cdcollect_small.png");
		win_importcd.Title = Catalog.GetString ("CDCollect: Edit CD");
      	
		/* Read GUI gconf properties */
		int win_height, win_width;
      		 
		/* Window size */
		win_height = (int) CDCollect.conf.getkey ( "gui/importcd_height", 150);
		win_width  = (int) CDCollect.conf.getkey ( "gui/importcd_width", 200);
		win_importcd.Resize( win_width, win_height);			
				
	}

	public void on_win_size_allocate (object o, Gtk.SizeAllocatedArgs args)
	{

		/* Save window position */
		int h,w;
        	
		win_importcd.GetSize(out w, out h);
        	
       	CDCollect.conf.setkey ("gui/importcd_height", h);
		CDCollect.conf.setkey ("gui/importcd_width", w);

	}
	
	private bool check()
	{
		/* Check for correct input name */
		if (entry_name.Text=="") {
			new ErrorDialog (Catalog.GetString ("Unvalid CD name, please enter a CD name"), win_importcd);
			return(false);
		}
				
		return(true);
	}

	private void on_ok_clicked (object o, EventArgs args)
	{
		if ( check() ) {
			/* Import a CD into the database */
			win_importcd.Hide ();
			
			cd.Name = entry_name.Text;
			cd.Keywords = entry_keywords.Text;
			cd.Date = dateedit.Time;
			cd.Icon = new CDIcon(img_icon.Pixbuf);

			EditCDEvent (cd);			

			win_importcd.Destroy();
		}
	}

	public void on_entry_name_activate (object o, EventArgs args)
	{
		on_ok_clicked (null, null);
	}

	public void on_entry_keywords_activate (object o, EventArgs args)
	{
		on_ok_clicked (null, null);
	}

	public void on_cancel_clicked (object o, EventArgs args)
	{
		win_importcd.Destroy();
	}

    public void on_delete_event (object o, DeleteEventArgs args) 
	{
		win_importcd.Destroy();
	}

	public void on_btn_icon_clicked (object o, EventArgs args)
	{
		fileselec.Filename = entry_icon.Text;
		fileselec.Modal = true;
		fileselec.Show ();
	}

	public void on_entry_icon_activate ( object o, EventArgs args )
	{
        /* Try to use cdicon */
        CDIcon icon = new CDIcon(entry_icon.Text, null);
    
        /* Show the icon */
        img_icon.Pixbuf = icon.pixbuf();
	}

	private void file_ok_event( object o, EventArgs args )
	{
		fileselec.Hide ();
		entry_icon.Text = fileselec.Filename;

        /* Try to use cdicon */
        CDIcon icon = new CDIcon(entry_icon.Text, null);
    
        /* Show the icon */
        img_icon.Pixbuf = icon.pixbuf();
	}

	private void file_cancel_event( object o, EventArgs args )
	{
		fileselec.Hide ();
	}


}

