/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Xml;

using Mono.Unix;

using Gtk;
using Gdk;
using Glade;
using GConf;
using Gnome; 


public class EditFileWindow : Gtk.Window
{

	/* Glade Widgets */
	[Glade.Widget] public Gtk.Window win_editfile;
	
	[Glade.Widget] public Gtk.Label lbl_filename;

	[Glade.Widget] public Gtk.Button btn_thumbnail;
	[Glade.Widget] public Gtk.Entry  entry_thumbnail;
	[Glade.Widget] public Gtk.Image  img_thumbnail;

	[Glade.Widget] public Gtk.TextView txtv_details;

	[Glade.Widget] public Gtk.Button btn_ok;
	[Glade.Widget] public Gtk.Button btn_cancel;
	


	/* Public variables */
        
	/* Private variables */
	private static Glade.XML gxml;
	private FileItem file;
	private FileSelection fileselec;
	private int thumbnail_size = 128; 
	
	/* Event to trigger when updating a File */
	public delegate void EditFileEventHandler(FileItem f,Pixbuf thumbnail);
	public event EditFileEventHandler EditFileEvent;
    
    /* Methods */
	public EditFileWindow (FileItem f, Pixbuf thumbnail, Gtk.Window parent) : base (IntPtr.Zero) 
	{
		/* Show Main Dialog using glade */
 		gxml = new Glade.XML (null, "gui.glade", "win_editfile", null);
			
		/* Connect functions to events */
		gxml.Autoconnect (this);
								
		win_editfile.TransientFor = parent;

		file = f;
		
		/* Set File properties */
		lbl_filename.Markup = "<big><b>" + f.Name + ":</b></big>";
		txtv_details.Buffer.Text = f.Info;
		img_thumbnail.Pixbuf = thumbnail;

		/* Create file selection */
		fileselec = new FileSelection (Catalog.GetString ("Thumbnail Selection"));	
		fileselec.OkButton.Clicked += file_ok_event;
        fileselec.CancelButton.Clicked +=new EventHandler (file_cancel_event);

		/* Show all widgets */
		win_editfile.Modal = true;
		win_editfile.ShowAll ();
           				
		/* Select focus */
		btn_cancel.GrabFocus();
        
	}

	public void prepare_gui()
	{
		/* Title */
		win_editfile.Icon = new Pixbuf(null, "cdcollect_small.png");
		win_editfile.Title = Catalog.GetString ("CDCollect: Edit File Details");
      	
		/* Read GUI gconf properties */
		int win_height, win_width;
      		 
		/* Window size */
		win_height = (int) CDCollect.conf.getkey ( "gui/editfile_height", 150);
		win_width  = (int) CDCollect.conf.getkey ( "gui/editfile_width", 200);
		win_editfile.Resize( win_width, win_height);		
				
	}

	public void on_win_size_allocate (object o, Gtk.SizeAllocatedArgs args)
	{

		/* Save window position */
		int h,w;
        	
		win_editfile.GetSize(out w, out h);
        	
       	CDCollect.conf.setkey ("gui/editfile_height", h);
		CDCollect.conf.setkey ("gui/editfile_width", w);

	}
	
	private bool check()
	{
		/* Nothing to check */
		return(true);
	}

	public void on_ok_clicked (object o, EventArgs args)
	{
		if ( check() ) {
			/* Import a CD into the database */
			win_editfile.Hide ();
			
			file.Info = txtv_details.Buffer.Text;
			EditFileEvent (file,img_thumbnail.Pixbuf);
			win_editfile.Destroy();
		}
	}

	public void on_cancel_clicked (object o, EventArgs args)
	{
		win_editfile.Destroy();
	}

    public void on_delete_event (object o, DeleteEventArgs args) 
	{
		win_editfile.Destroy();
	}

	public void on_btn_thumbnail_clicked (object o, EventArgs args)
	{
		fileselec.Filename = entry_thumbnail.Text;
		fileselec.Modal = true;
		fileselec.Show ();
	}

	public void on_entry_thumbnail_activate ( object o, EventArgs args )
	{    
        /* Show the icon */
        img_thumbnail.Pixbuf = import_thumbnail (entry_thumbnail.Text);
	}

	private void file_ok_event( object o, EventArgs args )
	{
		fileselec.Hide ();
		entry_thumbnail.Text = fileselec.Filename;

        /* Show the icon */
        img_thumbnail.Pixbuf = import_thumbnail (entry_thumbnail.Text);
	}

	private void file_cancel_event( object o, EventArgs args )
	{
		fileselec.Hide ();
	}

	private Pixbuf import_thumbnail (string file)
	{
		// Read a thumbnail image and resize it to desire length
		try {
			
			Pixbuf orig = new Pixbuf(file);
			Pixbuf thumb;
		
			// Scale the icon down
			if ( (orig.Width>thumbnail_size) || (orig.Height>thumbnail_size) ) {
		
				int nx,ny;
				if (orig.Height == orig.Width) {
		    		nx = thumbnail_size;
			    	ny = thumbnail_size;
				} else if (orig.Width > orig.Height) {
					nx = thumbnail_size;
					ny = (int)(((double)thumbnail_size)/((double)orig.Width)*((double)orig.Height));
				} else {
				    ny = thumbnail_size;
				    nx = (int)(((double)thumbnail_size)*((double)orig.Width)/((double)orig.Height));
				}
	
    			thumb =orig.ScaleSimple (nx,
    									 ny,
    									 Gdk.InterpType.Bilinear);
    			return(thumb);
    		
    		} else {
    		
    			return (orig);
    		
    		} 
		
		} catch {

			// Could not read file, do nothing

		}
		
		return (null);
		
	}

}

