/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections;
using System.IO;
using System.Xml;
using Gtk;
using Gdk;
using Mono.Unix;
using System.Text.RegularExpressions;

public enum FileItemType : int {
	Directory,
	File
}

public enum FileItemFind : int {
	NameInfo, /* name+info */
	Name,     /* name */ 
	Info,     /* info */
	Mime      /* mime */
}


public class FileItem {

	private int cd_key;
	public int CD_Key {
		set {
			cd_key = value;
		}
		get {
			return cd_key;
		}
	}

	private string path;
	public string Path {
		set {
			path = value;
		}
		get {
			return path;
		}
	}

	private string name;
	public string Name {
		set {
			name = value;
		}
		get {
			return name;
		}
	}

	private string info;
	public string Info {
		set {
			info = value;
		}
		get {
			return info;
		}
	}

	private string mime;
	public string Mime {
		set {
			mime = value;
		}
		get {
			return mime;
		}
	}

	private string location;
	public string Location {
		set {
			location = value;
		}
		get {
			return location;
		}
	}

	private int size;
	public int Size {
		set {
			size = value;
		}
		get {
			return size;
		}
	}
	
	private bool packed;
	public bool Packed {
		set {
			packed = value;
		}
		get {
			return packed;
		}
	}

	private FileItemType type;
	public FileItemType Type {
		set {
			type = value;
		}
		get {
			return type;
		}
	}

	public FileItem (int id, string p, string n, string i, int s, string m, string l, FileItemType t, bool pa)
	{
		cd_key    = id;
		path     = p;
		name     = n;
		info     = i;
		size     = s;
		mime     = m;
		type     = t;
		packed   = pa;
		location = l;
	}
		
 	public string FormatName ()
 	{
		string escaped = name;
		escaped = escaped.Replace("&", "&amp;");
		if (type==FileItemType.Directory) {
			if (packed) {
				return ("<i>" + escaped + "</i>");
			} else {
				return ("<b>" + escaped + "</b>");
			}
		} else {
			if (packed) {
				return ("<i>" + escaped + "</i>");
			} else {
				return (escaped);
			}
		}
	}

	public string ThumbKey ()
	{
		return (cd_key.ToString() + "_" + path);
	}

	public bool Find (string text, bool casesen, FileItemFind opt)
	{
	
	   string ctext, cname, cinfo, cmime;
	   
	   if (casesen) {
	       cname = name;
	       cinfo = info;
	       cmime = mime;
	       ctext = text;
	   } else {
	       cname = name.ToLower ();
	       cinfo = info.ToLower ();
	       cmime = mime.ToLower ();
	       ctext = text.ToLower ();
        }	       
	   
	   switch (opt) {
    	   case FileItemFind.Name:
    	       return (Regex.IsMatch(cname, ctext));
    	       	   
    	   case FileItemFind.Info:
    	       return (Regex.IsMatch (cinfo, ctext));
    	   
    	   case FileItemFind.Mime:
    	       return (Regex.IsMatch (cmime, ctext));

      	   case FileItemFind.NameInfo:
    	       if (Regex.IsMatch (cname, ctext)) {
    	           return (true);
    	       } else {
    	           return (Regex.IsMatch (cinfo, ctext));
    	       }
	   
	   }
	
	   return (false);
	}
	
}
