/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using Gtk;
using Gdk;

using System;
using System.Collections;

using Mono.Unix;


public class FilesView  : TreeView
{	
		
	/* The key of the showed cd */
	public int cdkey;
	public Hashtable files;
	
	//public Hashtable mimeicons;
	//Gtk.IconTheme icontheme;
	
	public int column_name = 0;
	public int column_info = 1;
	public int column_packed = 2;
	public int column_thumbkey = 3;
	
	/* Event to trigger when selecting a file */
	public delegate void FileSelectedEventHandler(int key, string path);
	public event FileSelectedEventHandler FileSelectedEvent;

	/* event to trigger when requesting a popup */
 	public delegate void PopupEventHandler(string widget);
	public event PopupEventHandler PopupEvent;

		
	public FilesView() : base(new TreeStore (	//typeof(Pixbuf),	    /* nano icon */ 
												typeof(string),     /* name */
												typeof(string), 	/* info */
												typeof(bool),       /* packed */
												typeof(string) ))	/* thumbkey */
	{

		/* Variables */	
		cdkey = -1; 
		//mimeicons = new Hashtable();
		//icontheme = new Gtk.IconTheme ();
			
		/* Set some properties for the treeview */
		RulesHint = true;
        HeadersVisible = false;
        
        /* Append columns in the treeview */
        //AppendColumn (Catalog.GetString ("Icon"), new CellRendererPixbuf(), "pixbuf", 0);
        AppendColumn (Catalog.GetString ("Name"), new CellRendererText(), "markup", column_name);
	
		/* Enable Search */
		EnableSearch = true;
		SearchColumn = column_name;

		// Create Popup Menu
		PopupMenu += new PopupMenuHandler (on_popup);
		
		/*
		filesview_popup = new FilesViewPopup();
		filesview_popup.FilesViewPopupEvent += new FilesViewPopup.FilesViewPopupEventHandler (on_popup_event);
		PopupMenu += new PopupMenuHandler (on_popup); 
		*/
		

		/* Associate events */
		RowActivated += new RowActivatedHandler (on_filesview_activate);
		//KeyPressEvent += new KeyPressEventHandler (on_keypress_event);

	}

/*
	private Pixbuf mime_icon (FileItem f)
	{
		return (null);

		// Check mimetype for directories
		if (f.Type==FileItemType.Directory) {
			//mime = "inode/directory";
			return (new Pixbuf(null,"mime-directory.png"));
		} else {
			//mime = f.Mime;
			return (null);
		}	
	
*/	
		/*
		string mime;
		Pixbuf icon;
		
		
		// Check mimetype for directories
		if (f.Type==FileItemType.Directory) {
//			mime = "x-directory/normal";
			mime = "inode/directory";
		} else {
			mime = f.Mime;
		}

		// Check if icon exists (and update mimeicons list)
		if (this.mimeicons.Contains(mime)) {
			icon = (Pixbuf) mimeicons[mime];
		} else {
			
			// Get icon from theme
			Gnome.Vfs.MimeType mimetype = new Gnome.Vfs.MimeType(mime);
			Trace.WriteLine ("Icon_name = '" + mimetype.Icon + "'");
						
			icon = icontheme.LoadIcon(mimetype.Icon,24,Gtk.IconLookupFlags.NoSvg);
			
			//if (icon.Height!=24) {
			//	icon = icon.ScaleSimple(24,24,Gdk.InterpType.Bilinear);
			//}
			
			this.mimeicons.Add(mime,icon);
		}
		
		return (icon);

		return(null);

	}
*/

	private TreeIter append_file (FileItem f)
	{
		TreeIter iter;

		/* This must be done using a thread notify event */		
		iter = (Model as TreeStore).AppendValues (
				//mime_icon (f),
				f.FormatName (),
				f.Info,
				f.Packed,
				f.ThumbKey ()
			);
				
		return(iter);
	}
	private TreeIter append_file (TreeIter iter, FileItem f)
	{
		TreeIter iter2;

		iter2 = (Model as TreeStore).AppendValues (
				iter, 
				//mime_icon (f),
				f.FormatName (),
				f.Info,
				f.Packed,
				f.ThumbKey ()
			);
				
		return(iter2);
	}

	public void Clear ()
	{
		cdkey = -1;
		
		(Model as TreeStore).Clear();		
	}

	public void fill (CDItem cd, Hashtable files)
	{				
		/* Copy data */
		cdkey = cd.Key;
		this.files = files;
				
		/* Disable sorting */
		//(Model as TreeModelSort).DefaultSortFunc = None;

		/* Start from root */
		fill_root ();
		
		/* Enable sorting */
		//(Model as TreeModelSort).ResetDefaultSortFunc ();

		/* Select row that was selected */
		select_path (cd.Selected);
	}
	
	public void select_path (string p)
	{
		if ( p != "" ) {
		
			TreePath path = new TreePath(p);
			
			/* Make sure all parent paths are expanded */
			ExpandToPath  (path);
			Selection.SelectPath (path);
			//ScrollToCell (path, GetColumn(0), true, 0.5F, 0.5F);
			//ScrollToCell (path, GetColumn(0), false, 0.5F, 0.5F);
			ScrollToCell (path, null, true, 0, 0);
		}
	}
	
	public void fill_root ()
	{
		TreeIter iter;
		int i = 0;
		bool done = false;
		string path;
				
		while (!done) {
		
			path = i.ToString();
			
			if (files.Contains(path)) {

				/* Insert file in root */
				iter = append_file ((FileItem)files[path]);
				
				/* Insert children */
				fill_child (iter, path + ":"); 
				
			} else {
				done = true;
			}
			
			i++;
		}
		
	}


	private void fill_child (TreeIter piter, string ppath) 
	{
		TreeIter iter;
		int i = 0;
		bool done = false;
		string path;

		while (!done) {

			path = ppath + i.ToString();

			if (files.Contains (path)) {
			
				/* Insert file */
				iter = append_file (piter, (FileItem)files[path]);

				/* Insert cildren */
				fill_child (iter, path + ":"); 

			} else {
				done = true;
			}

			i++;
		}
	}
	

	public void on_filesview_activate (object o, RowActivatedArgs args)
    {
    	/* Get the cellrenderer of the activated row */
    	if ( args.Column.CellRenderers[0].IsExpander ) {  	
	    	if ( args.Column.CellRenderers[0].IsExpanded ) {
    			CollapseRow (args.Path);
    		} else {
				ExpandRow (args.Path, false);
			}
		}    	
	}

	protected override bool OnButtonPressEvent(Gdk.EventButton e)
	{
		base.OnButtonPressEvent(e);
		if (e!=null) {
			if (e.Button==3) {
				//filesview_popup.Popup (get_selected_files (),e.Button,e.Time);
				PopupEvent ("filesview");
				return(true);
			}
		}		
		return(true);
	}

	public void on_popup (object o, PopupMenuArgs args)
	{
		PopupEvent ("filesview");
	}
	

	public ArrayList get_selected_files()
	{
		TreeModel m;
		ArrayList files_sel = new ArrayList ();
	 
		TreePath [] p = Selection.GetSelectedRows (out m);       
        				       
        for (int j=0; j<p.Length; j++) {					
			files_sel.Add ((FileItem)files[p[j].ToString()]);
		}
       
		return (files_sel);
	}
	
	public void update_file (FileItem f)
	{
		// Update view with file
		if (files.Contains (f.Path)) {
			files[f.Path] = f;
		}
		
		// Update tree with info
		TreeIter iter;
		(Model as TreeStore).GetIterFromString (out iter,f.Path);
		Model.SetValue (iter,column_info,f.Info);		
		
		// Update selection
		
	}
	
}

