/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Threading;
using System.Collections;
using System.Diagnostics;
using System.Xml;

using GLib;
using Gdk;
using Gtk;
using Glade; 
using GConf;
using Gnome; 

using Mono.Unix;


public class FindCollectionWindow : Gtk.Window
{
             
	/* Glade Widgets */
	[Glade.Widget] Gtk.Window win_find;

	[Glade.Widget] Gtk.CheckButton cbtn_name;
	[Glade.Widget] Gtk.ComboBox cbox_name;
	[Glade.Widget] Gtk.Entry  entry_name;
	[Glade.Widget] Gtk.CheckButton cbtn_case;

	[Glade.Widget] Gtk.CheckButton cbtn_size;
	[Glade.Widget] Gtk.ComboBox cbox_size;
	[Glade.Widget] Gtk.SpinButton sbtn_size;
	[Glade.Widget] Gtk.ComboBox cbox_bits;

	[Glade.Widget] Gtk.CheckButton cbtn_showcd;
	[Glade.Widget] Gtk.ScrolledWindow sw;
	
	[Glade.Widget] Gtk.HButtonBox hbtnbox;
	[Glade.Widget] Gnome.AppBar appbar;
		
	
	/* Public variables */
        
	/* Private variables */
	private static Glade.XML gxml;
    private static Gtk.TreeView tv_files;
    private static Gtk.TreeView tv_cds;	
    private FindOptions findopts;
		
	/* Event to trigger */
	/* Event to trigger when selecting a file */
	public delegate void FileSelectedEventHandler(int key, string path);
	public event FileSelectedEventHandler FileSelectedEvent;
	
	    
    /* Methods */        
	public FindCollectionWindow (Gtk.Window parent) : base (IntPtr.Zero) 
	{		

		/* Show Main Dialog using glade */
 		gxml = new Glade.XML (null, "gui.glade", "win_find", null);
			
		/* Connect functions to events */
		gxml.Autoconnect (this);    
		
		win_find.TransientFor = parent;

        /* Get Find Options */
        findopts = new FindOptions ();
        
		/* Prepare gui */
		prepare_gui ();    

		/* Entry text */
		entry_name.Text = findopts.text;
		sbtn_size.Value = findopts.size;
        
        /* Control find */
        control_dofind ();
                    
		/* Set it to modal */
		win_find.Modal = true;
		win_find.ShowAll ();

		/* Do not show Stop button */
		show_find_button ();
		
		/* Select focus */      											    																		    											
        entry_name.GrabFocus();

		/* Connect events (already done in glade) 
		CloseEvent += on_close_activate;
		StopEvent  += on_stop_activate;
		FindEvent  += on_find_activate; */

	}

	
		
	private void prepare_gui ()
	{
		/* Title */
		win_find.Icon = new Pixbuf(null, "cdcollect_small.png");
		win_find.Title = Catalog.GetString ("CDCollect: Find");
      	
		/* Read GUI gconf properties */
		int win_height, win_width;
      
		/* Window size */
		win_height = (int) CDCollect.conf.getkey ( "gui/find_height", 300);
		win_width  = (int) CDCollect.conf.getkey ( "gui/find_width", 200);
		win_find.Resize( win_width, win_height);
		
		/* Create tree views */
		create_tree_views (); 

		/* General do */
		cbtn_name.Active = findopts.doname;
		cbtn_size.Active = findopts.dosize;
		
		/* OptionMenus */
		cbox_name.Active = findopts.nameopt;
		cbox_size.Active = findopts.sizeopt;
        cbox_bits.Active = findopts.sizebit;

		/* Name Case */
		cbtn_case.Active = findopts.casesen;
		
		/* Show CD */
		cbtn_showcd.Active = findopts.showcd;
		
        /* Select the proper treeview (sw must be empty) */
	    if (cbtn_showcd.Active) {
            sw.Add (tv_cds);
        } else {
            sw.Add (tv_files);
        }
		
	}

    public void progress (float f, string msg)
    {
		appbar.ProgressPercentage = f;
		appbar.Progress.Text = msg;
    }
    public void push_appbar (string msg)
    {
	   appbar.Push (msg);
	}

	public void show_find_button ()
	{	   
		hbtnbox.Children[0].Visible = false;
		hbtnbox.Children[1].Visible = true;
    }		
	public void show_stop_button ()
	{	   
		hbtnbox.Children[0].Visible = true;
		hbtnbox.Children[1].Visible = false;
    }
    
    
	public void on_win_size_allocate (object o, Gtk.SizeAllocatedArgs args)
	{

		/* Save window position */
		int h,w;
        	
		win_find.GetSize(out w, out h);
        	
       	CDCollect.conf.setkey ("gui/find_height", h);
		CDCollect.conf.setkey ("gui/find_width", w);

	}

	public void on_name_changed (object o, EventArgs args)
    {			
		/* Copy the contents of the entry */
        findopts.text = entry_name.Text;
	}

	public void on_size_changed (object o, EventArgs args)
    {			
		/* Copy the contents of the entry */
        findopts.size = sbtn_size.Value;		
	}

    private void on_cbtn_showcd_toggled (object o, EventArgs args) 
	{
	    /* Show files or cd treeview */
        if (sw.Child!=null) {
            sw.Remove (sw.Child);
        }

	   if (cbtn_showcd.Active) {
	       sw.Add (tv_cds);
	   } else {
	       sw.Add (tv_files);
	   }
	   
	   sw.ShowAll ();
	   
	   //CDCollect.gconfClient.Set ( "/apps/CDcollect/prefs/find_showcd", cbtn_showcd.Active ? 1 : 0);
	   findopts.showcd = cbtn_showcd.Active;
	}

    public void on_find_activate (object o, EventArgs args) 
	{   	
		/* Show Stop button */
		show_stop_button ();
		
		/* Prepare Status Bar */
		appbar.Push (Catalog.GetString ("Searching CDs ..."));
		
		/* Get text and size values */
		findopts.text = entry_name.Text;
		findopts.size = sbtn_size.Value; 
		
	    /* Trigger the event 
        FindEvent (findopts); */
        
        find_files ();
	}

    private void find_files () 
	{
		/* Clear prior list */
		(tv_files.Model as ListStore).Clear();
		(tv_cds.Model as ListStore).Clear();
	    
        /* Save find properties */
        findopts.save_find_opts ();

		/* Execute query */
		Trace.WriteLine ("Executing query.");
		ArrayList files = CDCollect.db.find_files ( findopts.doname, 
													findopts.nameopt_in_str(), 
													findopts.text, 
													findopts.casesen, 
													findopts.dosize, 
													findopts.sizeopt_in_str(), 
													findopts.size_in_bytes());


		/* Get unique CDs */
		Trace.WriteLine ("Getting unique CDs in search results.");
		Hashtable cds = CDCollect.db.read_cds (files);
								
		/* Put files in window */
		Trace.WriteLine ("Writing results to find views.");

		/* FIXME: Stop sorting to speed up presentations not possible!! */
        if (sw.Child!=null) {
            sw.Remove (sw.Child);
        }
		create_tree_views ();		

		FileItem f;
		for (int i=0; i<files.Count; i++) {
			f = (FileItem)files[i];
			append_file ((CDItem)cds[f.CD_Key],f);
		}
		
		foreach (CDItem cd in cds.Values) {
			append_cd (cd);
		}			

		/* FIXME: should be able to add sorting again
		(tv_files.Model as ListStore).SetSortColumnId( 5, Gtk.SortType.Ascending);
		sort_column (2, 5, tv_files);
        (tv_cds.Model as ListStore).SetSortColumnId( 1, Gtk.SortType.Ascending);
		sort_column (0, 1, tv_cds); */
		on_cbtn_showcd_toggled (null, null);
							
		/* End finding */
		Trace.WriteLine ("Finishing find.");
		progress (0.0F, "");
		push_appbar ( String.Format (Catalog.GetString ("{0} Items found in {1} CDs."), files.Count, cds.Count));
		show_find_button ();

	}


    public void on_stop_activate (object o, EventArgs args) 
	{
		/* Show progress bar */
		appbar.ProgressPercentage = 0.0F;
		appbar.Progress.Text = "";
		appbar.Pop ();
		appbar.Push (Catalog.GetString ("Find stopped."));
		
		/* Show Find again */
		show_find_button ();
		
		/* Trigger the stopped event
		StopEvent (); */
	}

    public void on_close_activate (object o, EventArgs args) 
	{
		/* Trigger the close event
		CloseEvent (findopts); 

        if (sw.Child != null) {
            sw.Remove (sw.Child);
        }
		*/
	
		findopts.save_find_opts ();	
		win_find.Destroy ();
	}

    public void on_delete_event (object o, DeleteEventArgs args) 
	{
		/* Trigger the close event
		CloseEvent (findopts);
		
        if (sw.Child!=null) {
            sw.Remove (sw.Child);
        }
        */
		
		win_find.Destroy ();
	}
	
	public void Close ()
	{
	   on_close_activate (null, null);
	}
		

	public void on_cbtn_name_toggled (object o, EventArgs args) 
	{
	   /* Save option changed */
	   findopts.doname = cbtn_name.Active;
	   
	   /* Sensitive the find */
	   control_dofind ();       	   
	}
	public void on_cbtn_size_toggled (object o, EventArgs args) 
	{
	   /* Save option changed */
		findopts.dosize = cbtn_size.Active;

	   /* Sensitive the find */
	   control_dofind ();       	   
	}

	public void on_cbox_size_changed (object o, EventArgs args) 
	{
		findopts.sizeopt = cbox_size.Active;
	}

	public void on_cbox_bits_changed (object o, EventArgs args) 
	{
		findopts.sizebit = cbox_bits.Active;
	}
	public void on_cbox_name_changed (object o, EventArgs args) 
	{
		findopts.nameopt = cbox_name.Active;
	}
	public void on_cbtn_case_toggled (object o, EventArgs args) 
	{
		findopts.casesen = cbtn_case.Active;
	}


    private void control_dofind ()
    {
        /* Control doname */
	    if (cbtn_name.Active) {
	       entry_name.Sensitive = true;
	       cbox_name.Sensitive = true;
	       cbtn_case.Sensitive = true;
	    } else {
	       entry_name.Sensitive = false;
	       cbox_name.Sensitive = false;
	       cbtn_case.Sensitive = false;
        }
      
        /* Control dosize */
	    if (cbtn_size.Active) {
	       sbtn_size.Sensitive = true;
	       cbox_size.Sensitive = true;
	       cbox_bits.Sensitive = true;
	    } else {
	       sbtn_size.Sensitive = false;
	       cbox_size.Sensitive = false;
	       cbox_bits.Sensitive = false;
        }
      
        /* Control find button */
        if ( cbtn_name.Active || cbtn_size.Active ) {
            hbtnbox.Children[1].Sensitive = true;
        } else {
            hbtnbox.Children[1].Sensitive = false;
        }

    }

    private void create_tree_views ()
    {
    
		/* TreeView FILES widget */
		tv_files = new Gtk.TreeView ( new ListStore	(	
														typeof(int),		/* cd.Key */
														typeof(string),		/* cd.Name */ 
														typeof(string),		/* icon */
														typeof(string),		/* name */
														typeof(string),		/* path */
														typeof(string),		/* location */
														typeof(int) 		/* size */
													)
									);
		tv_files.RulesHint = true;
        tv_files.HeadersVisible = true;
        tv_files.HeadersClickable = true;
        
        TreeViewColumn tc;
        tc = tv_files.AppendColumn (Catalog.GetString ("Name"), new CellRendererText(), "markup", 3);
        tc.Resizable = true;
        tc.Clicked += on_column_name_click;
        tc = tv_files.AppendColumn (Catalog.GetString ("CD"), new CellRendererText(), "markup", 1);
        tc.Resizable = true;
        tc.Clicked += on_column_cd_click;
        tc = tv_files.AppendColumn (Catalog.GetString ("Location"), new CellRendererText(), "text", 5);	
        tc.Resizable = true;
        tc.Clicked += on_column_location_click;
        tc.SortIndicator = true;
				
		tv_files.HeadersClickable = true;
		
		/* Set Sorting in tree views (not here)
        (tv_files.Model as ListStore).SetSortColumnId( 5, Gtk.SortType.Ascending);
   		tv_files.GetColumn(2).SortIndicator = true;
		tv_files.GetColumn(2).SortOrder = Gtk.SortType.Ascending; */
		
		/* Associate events of treeview */
		tv_files.RowActivated += new RowActivatedHandler (on_row_activate);		

        /* Treeview CD widget */
        tv_cds = new Gtk.TreeView	( new ListStore	(	
														typeof(int),	/* key */
														typeof(string), /* name */
														typeof(string),	/* date */
														typeof(string)	/* keywords */
													)
									);
		tv_cds.RulesHint = true;
        tv_cds.HeadersVisible = true;
        tv_cds.HeadersClickable = true;
        
        tc = tv_cds.AppendColumn (Catalog.GetString ("Name"), new CellRendererText(), "markup", 1);
        tc.Resizable = true;
        tc.Clicked += on_cds_column_name_click;
        tc = tv_cds.AppendColumn (Catalog.GetString ("Date"), new CellRendererText(), "text", 2);
        tc.Resizable = true;
        tc.Clicked += on_cds_column_date_click;
        tc = tv_cds.AppendColumn (Catalog.GetString ("Keywords"), new CellRendererText(), "text", 3);
        tc.Resizable = true;
        tc.Clicked += on_cds_column_keywords_click;
				
		tv_cds.HeadersClickable = true;
		
		/* Set Sorting in tree views (not here )
        (tv_cds.Model as ListStore).SetSortColumnId( 1, Gtk.SortType.Ascending);
   		tv_cds.GetColumn(0).SortIndicator = true;
		tv_cds.GetColumn(0).SortOrder = Gtk.SortType.Ascending; */
   				
		/* Associate events of treeview */
		tv_cds.RowActivated += new RowActivatedHandler (on_row_activate);		
				        
	}

    /* Methods to control the column sort of tv_files and tv_cds */
	private void on_column_name_click (object o, EventArgs args)
	{
		sort_column (0, 3, tv_files);
	}
	private void on_column_cd_click (object o, EventArgs args)
	{
		sort_column (1, 1, tv_files);
	}
	private void on_column_location_click (object o, EventArgs args)
	{
		sort_column (2, 5, tv_files);
	}
	private void sort_column (int col, int id, Gtk.TreeView tv)
	{
		if (tv.GetColumn(col).SortIndicator == false) {
			(tv.Model as ListStore).SetSortColumnId( id, Gtk.SortType.Ascending);
			tv.GetColumn(col).SortOrder = Gtk.SortType.Ascending;
		} else {
			if (tv.GetColumn(col).SortOrder == Gtk.SortType.Ascending) {
				(tv.Model as ListStore).SetSortColumnId( id, Gtk.SortType.Descending);
				tv.GetColumn(col).SortOrder = Gtk.SortType.Descending;
			} else {
				(tv.Model as ListStore).SetSortColumnId( id, Gtk.SortType.Ascending);
				tv.GetColumn(col).SortOrder = Gtk.SortType.Ascending;
			}
		}
		for (int i=0; i<tv.Columns.Length; i++) {
    		tv.GetColumn(i).SortIndicator = false;
    	}
    			
		tv.GetColumn(col).SortIndicator = true;
	}
	private void on_cds_column_name_click (object o, EventArgs args)
	{
		sort_column (0, 1, tv_cds);
	}
	private void on_cds_column_date_click (object o, EventArgs args)
	{
		sort_column (1, 2, tv_cds);
	}
	private void on_cds_column_keywords_click (object o, EventArgs args)
	{
		sort_column (2, 3, tv_cds);
	}


    /* A row is activated in tv_files or tv_cds */
	public void on_row_activate (object o, RowActivatedArgs args)
	{
		TreeIter iter;
		string path;
		int key;

		/* Get selected cd and file */
		tv_files.Model.GetIter (out iter, args.Path);
		
		key = (int) tv_files.Model.GetValue(iter, 0);
		path = (string) tv_files.Model.GetValue(iter, 4);

		/* Mark this file as selected for that cd */
		((CDItem)CDCollect.db.cds[key]).Selected = path;

		/* Close find dialog */
		Close ();

		/* Event to select the cd and the corresponding file */
		FileSelectedEvent(key,path);		
				
	}

	private void append_file (CDItem cd, FileItem f)
	{
		(tv_files.Model as ListStore).AppendValues (
			cd.Key,
			cd.Name, 
			"",
			f.FormatName (),
			f.Path,
			f.Location,
			f.Size
		);				
	}	
	
	public void append_file (FileItem f)
	{
		(tv_files.Model as ListStore).AppendValues (
			f.CD_Key,
			"", 
			"",
			f.FormatName (),
			f.Path,
			f.Location,
			f.Size
		);				
	}		

	private void append_cd (CDItem cd)
	{
		(tv_cds.Model as ListStore).AppendValues (
			cd.Key,
			cd.Name, 
			cd.Date.Day + "/" + cd.Date.Month + "/" + cd.Date.Year,
            cd.Keywords
		);
				
	}	
		

}

