/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Threading;
using System.Collections;
using System.Diagnostics;
using System.Xml;

using GLib;
using Gdk;
using Gtk;
using Glade; 
using GConf;
using Gnome;

using Mono.Unix;



public class FindOptions {

    public bool doname;
    public string text;
	public bool casesen;
	public int nameopt;
    public bool dosize;
    public double size;
    public int sizeopt;
    public int sizebit;    
    public bool showcd;
        
    public FindOptions ()
    {
        read_find_opts ();
    }

	public string nameopt_in_str ()
	{
		switch (nameopt) {
			case 0: // Search file name
				return ("name");
			case 1: // Search info
				return ("info");
		}
		return("filename");		
    }

	public string sizeopt_in_str ()
	{
		switch (sizeopt) {
			case 0: /* > */
				return (">");
			case 1: /* < */
				return ("=");
			case 2: /* < */
				return ("<");
		}
		return("=");		
    }

	public int size_in_bytes ()
	{
		int size_bytes = 0;
		
        if (dosize) {
            switch (sizebit) {
                case 0: /* bytes */
                    size_bytes = (int)size;
                    break;
                case 1: /* Kbytes */
                    size_bytes = (int)(size * 1024.0F);
                    break;
                case 2: /* Mbytes */
                    size_bytes = (int)(size * 1024.0F * 1024.0F);
                    break;
                case 3: /* Gbytes */
                    size_bytes = (int)(size * 1024.0F * 1024.0F * 1024.0F);
                    break;
            }
        }
        
        return(size_bytes);
    }
        
    public void save_find_opts ()
    {
		CDCollect.conf.setkey ( "prefs/find_name", doname ? 1 : 0);
		CDCollect.conf.setkey ( "prefs/find_nameopt", nameopt);
		CDCollect.conf.setkey ( "prefs/find_namecase", casesen ? 1 : 0);
		CDCollect.conf.setkey ( "prefs/find_size", dosize ? 1 : 0);
		CDCollect.conf.setkey ( "prefs/find_sizeopt", sizeopt);
		CDCollect.conf.setkey ( "prefs/find_bit", sizebit);
		CDCollect.conf.setkey ( "prefs/find_text", text);
		CDCollect.conf.setkey ( "prefs/find_sizeval", size);
		CDCollect.conf.setkey ( "prefs/find_showcd", showcd ? 1 : 0);
	}

    public void read_find_opts ()
    {
            doname  = ((int)   CDCollect.conf.getkey ("prefs/find_name", 1)==1) ? true : false;
            text    = (string) CDCollect.conf.getkey ("prefs/find_text", "");
            casesen = ((int)   CDCollect.conf.getkey ("prefs/find_namecase", 0)==1) ? true : false;
            nameopt = (int)    CDCollect.conf.getkey ("prefs/find_nameopt", 0);
            dosize  = ((int)   CDCollect.conf.getkey ("prefs/find_size", 0)==1) ? true : false;
		    size    = (double) CDCollect.conf.getkey ("prefs/find_sizeval", 0.0);
            sizeopt = (int)    CDCollect.conf.getkey ("prefs/find_sizeopt", 0);
		    sizebit = (int)    CDCollect.conf.getkey ("prefs/find_bit", 0);
            showcd  = ((int)   CDCollect.conf.getkey ("prefs/find_showcd", 0)==1) ? true : false;
	}

    
}

