/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Threading;
using System.Collections;
using System.Xml;
using System.Diagnostics;

using Mono.Unix;

using GLib;
using Gtk;
using Gdk;
using Glade;
using GConf;
using Gnome; 


public class ImportCDProgressWindow : Gtk.Window
{
             
	/* Glade Widgets */
	[Glade.Widget] public Gtk.Window win_progress;
	[Glade.Widget] public Gtk.ProgressBar  pbar;
	[Glade.Widget] public Gtk.Label lbl;
	[Glade.Widget] public Gtk.Button btn_cancel;
	
	/* Public variables */
        
	/* Private variables */
	private static Glade.XML gxml;
	
	public int pbar_total;
	public int pbar_steps;
	public string pbar_text;
	public string action;
	
	public ThreadNotify stepnotify;
	public ThreadNotify actionnotify;

	/* Event to trigger when adding a CD */
	public delegate void CancelEventHandler();
	public event CancelEventHandler CancelEvent;


    /* Methods */        
	public ImportCDProgressWindow (Gtk.Window parent) : base (IntPtr.Zero) 
	{
		/* Show Main Dialog using glade */
 		gxml = new Glade.XML (null, "gui.glade", "win_progress", null);
			
		/* Connect functions to events */
		gxml.Autoconnect (this);    
		
		win_progress.TransientFor = parent;

      	/* Title */
		win_progress.Icon = new Pixbuf(null, "cdcollect_small.png");
		win_progress.Title = Catalog.GetString ("CDCollect: Importing CD");

		/* Widgets */
		pbar.Fraction = 0.0F;
		pbar.Text = "";
		lbl.Text = "";
						
		/* Create threads notify*/		
		stepnotify    = new ThreadNotify( new Gtk.ReadyEvent (step_notify) );
		actionnotify  = new ThreadNotify( new Gtk.ReadyEvent (action_notify) );
		
		/* Show all widgets */
		win_progress.Modal = true;
		win_progress.ShowAll ();

	}

	
	public void action_notify()
	{
		lbl.Text = action;
	}

	public void step_notify()
	{
		pbar.Fraction = (float)pbar_steps / (float)pbar_total;
		pbar.Text = pbar_text;
	}	
	
		
    private void on_cancel_clicked (object o, EventArgs args) 
	{
		/* Notify the cancel event */
		CancelEvent ();
	}

    public void on_delete_event (object o, DeleteEventArgs args) 
	{
        on_cancel_clicked (null, null);
	}

	public void Finish ()
	{
		win_progress.Destroy ();
	}
	

}

