/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using Gdk;
using Gtk;
using Glade;
using GConf;
using Gnome; 
using System.Xml;
using Mono.Unix;


public class ImportCDWindow : Gtk.Window
{
             
	// Glade Widgets
	[Glade.Widget] public Gtk.Window win_importcd;
	[Glade.Widget] public Gtk.Button btn_ok;
	[Glade.Widget] public Gtk.Button btn_cancel;
	[Glade.Widget] public Gtk.Entry  entry_name;
	[Glade.Widget] public Gtk.Entry  entry_keywords;
	
	[Glade.Widget] public Gnome.DateEdit  dateedit;
	
	[Glade.Widget] public Gtk.Button btn_icon;
	[Glade.Widget] public Gtk.Entry  entry_icon;
	[Glade.Widget] public Gtk.Image  img_icon;
	

	// Public variables
        
	// Private variables
	private static Glade.XML gxml;
	private string mount;
	
	private bool use_infoplugins, use_compplugins, compplugins_recursive, use_thumbs;
	private int key = -10;
	private FileSelection fileselec;

	private static System.Threading.Thread thread = null;			    
	private bool thread_finished = false;

	private ImportCDProgressWindow win_progress;

	// Event to trigger when adding a CD
	public delegate void ImportCDEventHandler(int key);
	public event ImportCDEventHandler ImportCDEvent;

	
    
    // Methods        
	public ImportCDWindow (string mount_point, bool info, bool comp, bool recursive, bool thumbs, Gtk.Window parent) : base (IntPtr.Zero) 
	{
		/* Show Main Dialog using glade */
 		gxml = new Glade.XML (null, "gui.glade", "win_importcd", null);
			
		/* Connect functions to events */
		gxml.Autoconnect (this);
						
		win_importcd.TransientFor = parent;

		/* Prepare gui */
		prepare_gui ();		
		
		mount = mount_point;
		use_infoplugins = info;
		use_compplugins = comp;
		compplugins_recursive = recursive;
		use_thumbs = thumbs;
		
		// Read CD name from somewhere
		entry_name.Text = get_cd_label ();

		/* Read CD creation date */
		DirectoryInfo dir = new DirectoryInfo (mount);
		if (dir.Exists) {
			dateedit.Time = dir.CreationTime;
		} else {
			dateedit.Time = new DateTime();
		}
				
		/* Select default icon */
		//iconentry.Filename = User.DirGet() + "cdcollect/cdcollect_small.png";
		img_icon.Pixbuf = new Pixbuf(null, "cdcollect_small.png");

		/* Create file selection */
		fileselec = new FileSelection (Catalog.GetString ("Icon Selection"));	
		fileselec.OkButton.Clicked += file_ok_event;
        fileselec.CancelButton.Clicked +=new EventHandler (file_cancel_event);
						                                                              		                                                              
		/* Show all widgets */
		win_importcd.Modal = true;
		win_importcd.ShowAll ();
           				
        
		/* Select focus */	
        entry_name.GrabFocus();
        
	}
				
	private void prepare_gui()
	{
		/* Title */
		win_importcd.Icon = new Pixbuf(null, "cdcollect_small.png");
		win_importcd.Title = Catalog.GetString ("CDCollect: Import CD");
      	
		/* Read GUI gconf properties */
		int win_height, win_width;
      		 
		/* Window size */
		win_height = (int) CDCollect.conf.getkey ( "gui/importcd_height", 150);
		win_width  = (int) CDCollect.conf.getkey ( "gui/importcd_width", 200);
		win_importcd.Resize( win_width, win_height);			
				
	}

	public void on_win_size_allocate (object o, Gtk.SizeAllocatedArgs args)
	{

		/* Save window position */
		int h,w;
        	
		win_importcd.GetSize(out w, out h);
        	
       	CDCollect.conf.setkey ("gui/importcd_height", h);
		CDCollect.conf.setkey ("gui/importcd_width", w);

	}
	
	private bool check()
	{
		
		/* Check for correct input name */
		if (entry_name.Text=="") {
			new ErrorDialog (Catalog.GetString ("Unvalid CD name, please enter a CD name"), win_importcd);
			return(false);
		}
							
		return(true);
	}

	private void on_ok_clicked (object o, EventArgs args)
	{
		if ( check() ) {

			/* Import a CD into the database */
			win_importcd.Hide ();

			win_progress = new ImportCDProgressWindow (win_importcd);
			win_progress.CancelEvent += on_cancel_event;

			/* Do the importing in its own thread */
			thread = new System.Threading.Thread (new System.Threading.ThreadStart(importcd_thread));
			thread_finished = false;
			thread.Start ();
				
			/* Wait for the thread to finish */
			while (thread_finished==false) {
				Gtk.Application.RunIteration (false);
			}

			/* Destroy progress window */
			win_progress.Finish ();

			/* Call the event (in main) to update displays with the new imported CD */
			ImportCDEvent (key);			
	

        }
    }

	private void importcd_thread ()
	{
		/* Prepare the progress window */
		win_progress.action = "Reading CD files.";
		win_progress.actionnotify.WakeupMain ();

		/* Count number of files */
		win_progress.pbar_total = CDCollect.db.count_files (mount);
		win_progress.pbar_steps = 0;
		win_progress.stepnotify.WakeupMain ();

		/* Scan the entire CD */
		CDItem cd;
		key = CDCollect.db.import_cd_step1 (mount, entry_name.Text, entry_keywords.Text, dateedit.Time, 
				img_icon.Pixbuf,  use_infoplugins, use_compplugins, compplugins_recursive,
				win_progress, out cd);
	
		// Check for errors 
		if (key>=0) {
			key = CDCollect.db.import_cd_step2 (cd, mount, entry_name.Text, entry_keywords.Text, dateedit.Time, 
					img_icon.Pixbuf,  use_infoplugins, use_compplugins, compplugins_recursive,
					use_thumbs, win_progress);		
		}	
		
		/* Done */
		win_progress.pbar_steps = win_progress.pbar_total;
		win_progress.stepnotify.WakeupMain ();


		/* Notify the end of scaning */
		thread_finished = true;
	}
    


	public void on_cancel_event ()
	{

		/* Suspend the importing */
		if (thread != null) {
		  if (thread.IsAlive) {
		      thread.Suspend ();
		  }
		}

		/* Ask the user to really stop */
		YesNoDialog dialog = new YesNoDialog	(	
													null, 
													Catalog.GetString ("Are you sure you want to cancel the importing?"), 
													win_progress
												);

		if (dialog.GetAnswer () == true) {
		
			/* Cancel the importing */
    		if (thread != null) {
   				thread.Abort ();
    		}

            // Delete imported cd from database
			Console.WriteLine ("Delete imported CD if key!=1 (key=" + key.ToString() + ")");
            if (key>=0) {
                Console.WriteLine (String.Format (Catalog.GetString ("Deleting imported cd with key = {0}"),key));
                CDCollect.db.delete_cd_and_files (key);
            }
            
			/* Notify the main event that the thread is finished */
			thread_finished = true;
			
    		
    	} else {    	

			/* Resume the thread */
			if (thread != null) {
				thread.Resume ();
			}

    	}

	}

	public void on_entry_name_activate (object o, EventArgs args)
	{
		on_ok_clicked( null, null);
	}

	public void on_entry_keywords_activate (object o, EventArgs args)
	{
		on_ok_clicked( null, null);
	}

	public void on_cancel_clicked (object o, EventArgs args)
	{
		win_importcd.Destroy();
	}

    public void on_delete_event (object o, DeleteEventArgs args) 
	{
		win_importcd.Destroy();
	}

	public void on_btn_icon_clicked (object o, EventArgs args)
	{
		fileselec.Filename = entry_icon.Text;
		fileselec.Modal = true;
		fileselec.Show ();
	}

	public void on_entry_icon_activate ( object o, EventArgs args )
	{
        /* Try to use cdicon */
        CDIcon icon = new CDIcon(entry_icon.Text, null);
    
        /* Show the icon */
        img_icon.Pixbuf = icon.pixbuf();
	}

	private void file_ok_event( object o, EventArgs args )
	{
		fileselec.Hide ();
		entry_icon.Text = fileselec.Filename;

        /* Try to use cdicon */
        CDIcon icon = new CDIcon(entry_icon.Text, null);
    
        /* Show the icon */
        img_icon.Pixbuf = icon.pixbuf();
	}

	private void file_cancel_event( object o, EventArgs args )
	{
		fileselec.Hide ();
	}



	private string get_cd_label ()
	{
		FileStream fdev;
		int r = 0;
	
		/* Open the device */
		try {
			fdev = new FileStream (CDCollect.conf.default_cd_device, 
				System.IO.FileMode.Open, System.IO.FileAccess.Read);
		} catch {
			Console.WriteLine ("Can not open device " + CDCollect.conf.default_cd_device);
			return("Unknown");
		}	
		
		/* Read 32 bytes from offset 32808 */
		byte [] buf = new byte[33];
		try {
			fdev.Seek (32808, SeekOrigin.Begin);
			r = fdev.Read (buf, 0, 32);
		} catch (System.Exception e) {
			Console.WriteLine ("Can not read device " + CDCollect.conf.default_cd_device);
			return ("Unknown");
		}
		
		/* Copy buf into a char array  */
		try {
			char [] c = new char[r+1];
			for (int i=0; i<r; i++) {
				c[i] = (char)buf[i];
			}
			c[r] = '\0';
		
			/* Remove trailing spaces & '\0'*/
			int n = r;
			while ( (c[n]==' ') || (c[n]=='\0') ){
				n--;
			}
			char [] cc = new char[n+2];
			for (int j=0; j<=n; j++) {
				cc[j] = c[j];
			}
			cc[n+1] = '\0';		
		
			String s = new String(cc);
			return(s);
		} catch {
			return ("Unknown");
		}
		
	}
		
}

