/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Threading;
using System.Collections;
using System.Xml;
using System.Diagnostics;
using System.Text.RegularExpressions;

using Mono.Unix;

using GLib;
using Gtk;
using Gdk;
using Glade;
using GConf;
using Gnome;
using Gnome.Vfs;


public class InfoPluginTestWindow : Gtk.Window
{
             
	/* Glade Widgets */
	[Glade.Widget] public Gtk.Window win_exprtest;
	
	[Glade.Widget] public Gtk.Label  lbl_info;

	[Glade.Widget] public Gtk.Entry  entry_file;
	[Glade.Widget] public Gtk.Button btn_file;
	
	[Glade.Widget] public Gtk.Button btn_close;
	
	[Glade.Widget] public Gtk.TextView tv_out1;
	[Glade.Widget] public Gtk.TextView tv_out2;
	
	/* Public variables */
        
	/* Private variables */
	private static Glade.XML gxml;
	private FileSelection fileselec;
	private InfoPluginItem plug;
	//private static InfoMimeType infomime;
	InfoPluginWindow parent;
	
	
    /* Methods */        
	public InfoPluginTestWindow (InfoPluginItem p, InfoPluginWindow parent) : base (IntPtr.Zero) 
	{
		/* Show Main Dialog using glade */
 		gxml = new Glade.XML (null, "gui.glade", "win_exprtest", null);
			
		/* Connect functions to events */
		gxml.Autoconnect (this);    
		
		/* Prepare gui */
		prepare_gui ();

		win_exprtest.TransientFor = parent;


		plug = p;				
		lbl_info.Markup = String.Format (Catalog.GetString ("<b>Select file to test Regular Expression for mime '{0}':</b>"),plug.Mime);

        /* Create local variables */        
  		//infomime = new InfoMimeType();

		/* Create file selection */
		fileselec = new FileSelection (Catalog.GetString ("File Selection"));	
		fileselec.OkButton.Clicked += file_ok_event;
        fileselec.CancelButton.Clicked +=new EventHandler (file_cancel_event);

		this.parent = parent;
		entry_file.Text = parent.testfilename;

		/* Show all widgets */
		win_exprtest.Modal = true;
		win_exprtest.ShowAll ();

	}

	private void prepare_gui()
	{
		/* Title */
		win_exprtest.Icon = new Pixbuf(null, "cdcollect_small.png");
		win_exprtest.Title = Catalog.GetString ("CDCollect: Test Regular Expression");
      
		/* Read GUI gconf properties */
		int win_height, win_width;
      		 
		/* Window size */
		win_height = (int) CDCollect.conf.getkey ( "gui/exprtest_height", 300);
		win_width  = (int) CDCollect.conf.getkey ( "gui/exprtest_width", 200);
		win_exprtest.Resize( win_width, win_height);
	}
	
	public void on_win_size_allocate (object o, Gtk.SizeAllocatedArgs args)
	{
		/* Save window position */
		int h,w;
        	
		win_exprtest.GetSize(out w, out h);
        	
       	CDCollect.conf.setkey ("gui/exprtest_height", h);
		CDCollect.conf.setkey ("gui/exprtest_width", w);
	}
	
    public void on_btn_try_clicked (object o, EventArgs args) 
	{
        /* Try the plugin */
        try_plugin ();
	}

    public void on_btn_close_clicked (object o, EventArgs args) 
	{
		parent.testfilename = entry_file.Text;
        win_exprtest.Hide ();
        win_exprtest.Destroy ();
	}

    public void on_win_delete_event (object o, DeleteEventArgs args) 
	{
		parent.testfilename = entry_file.Text;
        win_exprtest.Hide ();
        win_exprtest.Destroy ();
	}
			
	public void on_btn_file_clicked (object o, EventArgs args)
	{
		fileselec.Filename = entry_file.Text;
		fileselec.Modal = true;
		fileselec.Show ();
	}

	public void on_entry_file_activate ( object o, EventArgs args )
	{
        /* Try the plugin */
        try_plugin ();
	}

	private void file_ok_event( object o, EventArgs args )
	{
		fileselec.Hide ();
		entry_file.Text = fileselec.Filename;
		
		/* try the plugin */
		try_plugin ();
	}

	private void file_cancel_event( object o, EventArgs args )
	{
		fileselec.Hide ();
	}
	
	
	public bool try_plugin ()
	{
		string filename = entry_file.Text;
	
		/* File exists ? */	
		if (!System.IO.File.Exists (filename)) {
            new ErrorDialog (Catalog.GetString ("The input file does not exists."));
			return(false);
		}
	
		/* Program exists ? */
		if (!System.IO.File.Exists (plug.Program)) {
            new ErrorDialog (Catalog.GetString ("The file program does not exists."));
			return(false);
		}
		
		/* Check that file mimetype is ok */
		//string mime = infomime.get_mime_type (filename);
		string mime = Gnome.Vfs.MimeType.GetMimeTypeForUri("file://" + filename);
		if (mime!=plug.Mime) {
            new ErrorDialog (String.Format (
       			Catalog.GetString ("The file mime type is '{0}'\nIt does not match '{1}'."),mime,plug.Mime
            	)
            );
   			return(false);
		}
		
		
		
		string saveargs = plug.Args.Replace("{0}","\"{0}\"");
			
		/* Execute the external program to get the mime info */
		System.Diagnostics.Process proc = new System.Diagnostics.Process();
		
		proc.EnableRaisingEvents = false;
		proc.StartInfo.UseShellExecute = false;
		proc.StartInfo.FileName = plug.Program;
		proc.StartInfo.Arguments = String.Format (saveargs, filename);
		proc.StartInfo.RedirectStandardOutput = true;
		
		proc.Start();
		proc.WaitForExit();

		/* Get raw output and format if using regular expressions */
		string rawout = proc.StandardOutput.ReadToEnd ();
		
		/* Change markups for pango */			
		rawout = Regex.Replace( rawout, "&", "&amp;");
		rawout = Regex.Replace( rawout, "<", "&lt;");
		rawout = Regex.Replace( rawout, ">", "&gt;");

		tv_out1.Buffer.Text = rawout;
					
		if (plug.RegexInput!="") {
				
				if (plug.RegexCase) {
					rawout = Regex.Replace( rawout, plug.RegexInput, plug.RegexPattern);
				} else {
					rawout = Regex.Replace( rawout, plug.RegexInput, plug.RegexPattern, RegexOptions.IgnoreCase);
				}
								
				tv_out2.Buffer.Text = rawout;
		}
		
		return(true);
		
		
	}

}

