/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Xml;
using System.Text.RegularExpressions;

using Mono.Unix;

using Gtk;
using Gdk;
using Glade;
using GConf;
using Gnome; 


public class InfoPluginWindow : Gtk.Window
{

	/* Glade Widgets */
	[Glade.Widget] public Gtk.Window win_infoplugin;

	[Glade.Widget] public Gtk.Button btn_ok;
	[Glade.Widget] public Gtk.Button btn_cancel;
	[Glade.Widget] public Gtk.Button btn_test;
		
	[Glade.Widget] public Gtk.Entry entry_mime;
	[Glade.Widget] public Gtk.Entry entry_program;
	[Glade.Widget] public Gtk.Entry entry_args;
	
	[Glade.Widget] public Gtk.CheckButton cbtn_regexp; 
	
	[Glade.Widget] public Gtk.Label lbl_regexp;
	[Glade.Widget] public Gtk.Label lbl_input;
	[Glade.Widget] public Gtk.Label lbl_pattern;

	[Glade.Widget] public Gtk.TextView entry_input;
	[Glade.Widget] public Gtk.TextView entry_pattern;
	
	[Glade.Widget] public Gtk.CheckButton cbtn_case; 
	
	[Glade.Widget] public Gtk.HBox hb_case; 
	[Glade.Widget] public Gtk.Table table_regexp; 
	
		
	/* Public variables */
        
	/* Private variables */
	private static Glade.XML gxml;
	private string oldmime;
	private FileSelection fileselec;
		
	/* Event to trigger when adding a Item */
	public delegate void AddEventHandler(InfoPluginItem item, string oldmime);
	public event AddEventHandler AddEvent;

   	public string testfilename;	
    
    
    /* Methods */        
	public InfoPluginWindow (InfoPluginItem infoitem, Gtk.Window parent) : base (IntPtr.Zero) 
	{
		/* Show Main Dialog using glade */
 		gxml = new Glade.XML (null, "gui.glade", "win_infoplugin", null);
			
		/* Connect functions to events */
		gxml.Autoconnect (this);
				
		win_infoplugin.TransientFor = parent;

		/* Create file selection */
		fileselec = new FileSelection (Catalog.GetString ("Program Selection"));	
		fileselec.OkButton.Clicked += file_ok_event;
        fileselec.CancelButton.Clicked +=new EventHandler (file_cancel_event);
		
		/* Set InfoPlugin properties */
		oldmime = infoitem.Mime;
		testfilename = "";
		
		entry_mime.Text = infoitem.Mime;
		entry_program.Text = infoitem.Program;
		entry_args.Text = infoitem.Args;
		
		
		if (infoitem.RegexInput=="") {
			cbtn_regexp.Active   = false;
			regexp_controls_visible (false);			
		} else {
			cbtn_regexp.Active   = true;
			regexp_controls_visible (true);
			entry_input.Buffer.Text = Regex.Escape (infoitem.RegexInput);
			entry_pattern.Buffer.Text = Regex.Escape (infoitem.RegexPattern);
			cbtn_case.Active = infoitem.RegexCase;
		}		

		/* Show all widgets */
		win_infoplugin.Modal = true;
		win_infoplugin.ShowAll ();
           				
		/* Prepare gui */
		prepare_gui ();

		if (infoitem.RegexInput=="") {
			regexp_controls_visible (false);			
		} else {
			regexp_controls_visible (true);
		}		

		/* Select focus */           											    																		    											
        btn_cancel.GrabFocus();
        
	}

    /* Methods */        
	public InfoPluginWindow (Gtk.Window parent) : base (IntPtr.Zero) 
	{
		/* Show Main Dialog using glade */
 		gxml = new Glade.XML (null, "gui.glade", "win_infoplugin", null);
			
		/* Connect functions to events */
		gxml.Autoconnect (this);
										
		win_infoplugin.TransientFor = parent;

		/* Create file selection */
		fileselec = new FileSelection (Catalog.GetString ("Program Selection"));	
		fileselec.OkButton.Clicked += file_ok_event;
        fileselec.CancelButton.Clicked +=new EventHandler (file_cancel_event);

		/* Set InfoPlugin properties */
		oldmime = "";
		testfilename = "";
		
		/* Show all widgets */
		win_infoplugin.Modal = true;
		win_infoplugin.ShowAll ();
           				
		/* Prepare gui */
		prepare_gui ();

		regexp_controls_visible (false);

		/* Select focus */           											    																		    											
        entry_mime.GrabFocus();

                
	}

	private void prepare_gui()
	{
		/* Title */
		win_infoplugin.Icon = new Pixbuf(null, "cdcollect_small.png");
		win_infoplugin.Title = Catalog.GetString ("CDCollect: Information Plugin");

		/* Read GUI gconf properties */
		int win_height, win_width;
      		 
		/* Window size */
		win_height = (int) CDCollect.conf.getkey ("gui/infoplugin_height", 150);
		win_width  = (int) CDCollect.conf.getkey ("gui/infoplugin_width", 200);
		win_infoplugin.Resize( win_width, win_height);
	}

	public void on_win_size_allocate (object o, Gtk.SizeAllocatedArgs args)
	{
		/* Save window position */
		int h,w;
        	
		win_infoplugin.GetSize(out w, out h);
        	
       	CDCollect.conf.setkey ("gui/infoplugin_height", h);
		CDCollect.conf.setkey ("gui/infoplugin_width", w);
	}
	
	private bool check()
	{
		/* Check for correct input mime */
		if (entry_mime.Text=="") {
			new ErrorDialog (Catalog.GetString ("Unvalid mime, please enter a valid mime type."), win_infoplugin);
			return(false);
		}
		/* Check for correct program name */
		if (entry_program.Text=="") {
			new ErrorDialog (Catalog.GetString ("Unvalid program name, please enter a valid external command."), win_infoplugin);
			return(false);
		}
		/* Check for correct program name */
		if ( ! System.IO.File.Exists(entry_program.Text) ) {
			new ErrorDialog (Catalog.GetString ("Unvalid program name, please enter a valid external command."), win_infoplugin);
			return(false);
		}
		return(true);
	}

	public void on_ok_clicked (object o, EventArgs args)
	{
		InfoPluginItem plug;
		
		if ( check() ) {

			/* Import a CD into the database */
			win_infoplugin.Hide ();

			if (cbtn_regexp.Active) {		
				plug = new InfoPluginItem(
					entry_mime.Text, entry_program.Text, entry_args.Text, 				
					Regex.Unescape (entry_input.Buffer.Text), Regex.Unescape (entry_pattern.Buffer.Text),
					cbtn_case.Active
				);
			} else {
				plug = new InfoPluginItem (entry_mime.Text, entry_program.Text, entry_args.Text, "", "", false); 	
			}			
			
			AddEvent (plug, oldmime);

			win_infoplugin.Destroy();
		}
	}

	public void on_cancel_clicked (object o, EventArgs args)
	{
		win_infoplugin.Destroy();
	}

	public void on_test_clicked (object o, EventArgs args)
	{
		InfoPluginItem p;

		if (cbtn_regexp.Active) {		
			p = new InfoPluginItem(
				entry_mime.Text, entry_program.Text, entry_args.Text, 				
				Regex.Unescape (entry_input.Buffer.Text), Regex.Unescape (entry_pattern.Buffer.Text),
				cbtn_case.Active
			);
			
			/* Open the test window */
			new InfoPluginTestWindow (p, this);
			
			
		}
	}

    public void on_delete_event (object o, DeleteEventArgs args) 
	{
		win_infoplugin.Destroy();
	}

	public void on_btn_browse_clicked (object o, EventArgs args)
	{
		fileselec.Filename = entry_program.Text;
		fileselec.Modal = true;
		fileselec.Show ();
	}

	private void file_ok_event( object o, EventArgs args )
	{
		entry_program.Text = fileselec.Filename;
		fileselec.Hide ();
	}

	private void file_cancel_event( object o, EventArgs args )
	{
		fileselec.Hide ();
	}

	public void on_useregexp_toggled (object o, EventArgs args)
	{
		if ( ! cbtn_regexp.Active ) {
			regexp_controls_visible (false);
		} else {
			regexp_controls_visible (true);
		}		

	}

	private void regexp_controls_visible(bool visib)
	{
		if (!visib) {
			lbl_regexp.Visible = false;
			cbtn_case.Visible  = false;
			lbl_input.Visible  = false;
			entry_input.Visible = false;
			lbl_pattern.Visible = false;
			entry_pattern.Visible = false;
			hb_case.Visible = false;
			table_regexp.Visible = false;
			btn_test.Visible = false;

			/* Make window smaller */
			int w  = (int) CDCollect.conf.getkey ("gui/infoplugin_width", 200);
			win_infoplugin.Resize (w, 245);
			
		} else {
			lbl_regexp.Visible = true;
			cbtn_case.Visible = true;
			lbl_input.Visible  = true;
			entry_input.Visible = true;
			lbl_pattern.Visible = true;
			entry_pattern.Visible = true;
			hb_case.Visible = true;
			table_regexp.Visible = true;
			btn_test.Visible = true;
		}
		
	}
}

