/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections;
using System.Diagnostics;
using System.Xml;

using Mono.Unix;

using Gtk;
using Gdk;
using GLib;
using Glade;
using Gnome;


public class CDCollect : Gnome.Program
{
             
	// Glade Widgets
	[Glade.Widget] public HPaned hpanel;
	[Glade.Widget] public VPaned vpanel;
	[Glade.Widget] public Gnome.App win_main;
	                
	[Glade.Widget] public Gnome.AppBar appbar;

	[Glade.Widget] public TreeView tv;

	[Glade.Widget] public ScrolledWindow sw_cds;
	[Glade.Widget] public ScrolledWindow sw_files;

	[Glade.Widget] public Gtk.Entry entry_search;

	[Glade.Widget] public Gtk.Toolbar tb_main;

	[Glade.Widget] public Viewport  vp_details;
	[Glade.Widget] public Gtk.HBox hbox_details;
	[Glade.Widget] public Gtk.Label lbl_info;
	[Glade.Widget] public Gtk.Image img_thumb;
	

	// Private variables
	private CDView cdview;
	private FilesView filesview;
	private FilesViewPopup filesview_popup;
	private static Glade.XML gxml;
	//private static bool cdview_selection_handler = true;
    private static FindCollectionWindow win_find;
																
	// Public variables
	public static CDDatabase db;
	public static Conf conf;
	
                       
	// Methods        
	public static void Main (string[] args)
	{
	    	new CDCollect (args);
	}

	public CDCollect (string[] args) : base ("CDCollect", "", Gnome.Modules.UI, args)
	{
	
   		int gotocd = -1;
   		string gotofile = "";
 	
		// Start configuration object
		Trace.WriteLine ("Starting gconf object.");
		conf = new Conf ();
	 		
   		// Initialise international strings
		Trace.WriteLine ("Initializing international strings.");
   		Catalog.Init ("cdcollect", conf.gnome_locale_dir);
   		   		
   		// Check if any other instance of cdcollect is running
		Trace.WriteLine ("Checking for another instance of cdcollect (FIXME).");

		// FIXME: bug in mutex class that keeps the mutes on the $HOME/.wapi folder
   		System.Threading.Mutex mlock;
		bool newInstance = false;        

		mlock = new System.Threading.Mutex(false, "CDCollect:" + User.DirGet(), out newInstance);

        //if (!newInstance) {
		newInstance = true;
		if (newInstance==false) {
        
            /* Notify the user CDCollect is running already and stop */
            new ErrorDialog(
            	Catalog.GetString ("There is an instance of CDCollect already running.\n") +
            	Catalog.GetString ("Please use the running instance instead of opening a new one."));

                            
        } else {

/*
			string mime = Gnome.Vfs.MimeType.GetMimeTypeForUri("file:///home/jrh/tmp/mp3/various");
			Console.WriteLine ("mime = '" + mime + "'");

			Gnome.Vfs.MimeType m = 	new Gnome.Vfs.MimeType (mime);
			Console.WriteLine ("Description = '" + m.Description + "'");
			Console.WriteLine ("Name = '" + m.Name + "'");
			Console.WriteLine ("Icon = '" + m.Icon + "'");
*/

            /* Check if sqlite v2 is used */
    		if ( System.IO.File.Exists(conf.dbname_v2) ) {
                new ErrorDialog (Catalog.GetString ("CDCollect now uses a new database format!!\n") +
                                 Catalog.GetString ("Unfortunately it is incompatible with the previous format.\n\n") +
                	             Catalog.GetString ("To convert your database to the new version follow these steps.\n") +
                                 Catalog.GetString ("First, you need the SQLITE command-line shells for both version 2.8 and 3.0.\n") +
                                 Catalog.GetString ("Then enter these commands in a terminal:\n\n") +
                                 Catalog.GetString ("   <i>cd $HOME/.gnome2/cdcollect/</i>\n") +
                                 Catalog.GetString ("   <i>cp cds.db cds.db_backup</i>\n") +
                                 Catalog.GetString ("   <i>sqlite cds.db .dump | sqlite3 cds3.db</i>\n") +
                                 Catalog.GetString ("   <i>rm cds.db</i>\n\n") +
                                 Catalog.GetString ("For more information, please go to the CDCollect FAQ at http://cdcollect.sourceforge.net/faq.php")
               	             );

    		} else {
          

                // Create Database
                Trace.WriteLine ("Creating Database.");
                db = new CDDatabase (conf.dbname,conf.thumbsdir);
                //db = new CDDatabase (conf.dbname_v2,conf.thumbsdir);

                // Check input args
                if (args.Length>0) {
 		
                    for (int i=0; i<args.Length; i++) {
 			
                        if (args[i]=="--cd") {
                            i++;
                            gotocd = Convert.ToInt32 (args[i]);
                            Trace.WriteLine ("Going to CD " + gotocd.ToString());
                        }
                        if (args[i]=="--file") {
                            i++;
                            gotofile = args[i];
                            Trace.WriteLine ("Going to path '" + gotofile + "'");
                        }
                        if (args[i]=="--dump-cds") {
                            Console.WriteLine (db.dump_cds ());
                            System.Environment.Exit (0);
                        }
                    }
                }

                // Read Sorting Type
                string cdview_sorting = (string) conf.getkey ("prefs/sort", "name");	

                // Create CD view
                Trace.WriteLine ("Creating CD view widget.");
                cdview = new CDView(cdview_sorting);
		
                // Associate events to cdview
                cdview.SelectionChangeEvent += on_cdview_selection_change;
                cdview.DeleteCDsEvent += on_cdview_delete_selected;
                cdview.EditCDEvent += on_cdview_edit;

                // Create FilesView
                Trace.WriteLine ("Creating FilesView widget.");
                filesview = new FilesView();

                // Associate events to filesview
                filesview.Selection.Changed += new EventHandler (on_filesview_selection_change);
                filesview.PopupEvent += on_filesview_popup;
                //filesview.EditFileEvent += on_filesview_edit;
		
                // Create Popup Menu for filesview and details
                filesview_popup = new FilesViewPopup();
                filesview_popup.FilesViewPopupEvent += new FilesViewPopup.FilesViewPopupEventHandler (on_filesview_popup_click);

                // FIXME: Need an event to scroll through filesview 
                filesview.FileSelectedEvent += find_file_selected;	   

                //string msg = String.Format (Catalog.GetString ("There are {0} CDs and {1} Files in the DB"), db.cds.Count, db.Nfiles () );
			
                // Create Main Window using Glade 
                Trace.WriteLine ("Creating main window.");
                gxml = new Glade.XML (null, "gui.glade", "win_main", null);
			
                // Connect functions to events
                gxml.Autoconnect (this);			
						
                // Add event to popup file menu 			
                vp_details.ButtonPressEvent += new ButtonPressEventHandler (on_details_buttonpress_event);

                // prepare gui
                prepare_gui();        
		
                // Appbar basic message
                appbar.Default = Catalog.GetString ("Ready");
		
                // Add cdview into main window
                sw_cds.Add (cdview);
		
                /* Add filesview into main window */
                sw_files.Add (filesview);			 

                /* Add message to appbar */
                //appbar.Push (msg);
													
                /* Select focus on search entry */
                entry_search.GrabFocus();
        
                /* Show all widgets */
                win_main.ShowAll ();
			

	       		/* Go to signaled CD and File */
	       		if (gotocd!=-1) {

	       			cdview.Selection.UnselectAll ();
	       			cdview.select_cd(gotocd);
		
	       			/* Select current path */
	      			filesview.CollapseAll ();
    				filesview.select_path (gotofile);
				
	       		}

    			/* Main gnome loop */
	      		Trace.WriteLine ("Running main gnome loop.");			
	      		Run ();

            }
		
		}
	
		// keep the mutex reference alive until termination of the program ???
		//System.GC.KeepAlive (mlock);

		// Release the mutex
		//Console.WriteLine ("releasing mutex");
		mlock.ReleaseMutex ();
	}

	
	private void prepare_gui()
	{
      	
      	win_main.Icon = new Pixbuf (null, "cdcollect.png");
      	
		/* Read GUI gconf properties */
		int hpanel_pos, vpanel_pos;
		int win_main_height, win_main_width;
      		 
		/* Main window size */
		win_main_height = (int) conf.getkey ("gui/main_height", 200);
		win_main_width  = (int) conf.getkey ("gui/main_width", 200);
		win_main.Resize( win_main_width, win_main_height);			
				
		/* HPanel position */
       	hpanel_pos = (int) conf.getkey ("gui/main_hpanel_pos", 200);
		hpanel.Position = hpanel_pos;
		
      	/* VPanel position */
        vpanel_pos = (int) conf.getkey ("gui/main_vpanel_pos", 200);
		vpanel.Position = vpanel_pos;
     	      		      	      		      		
	}
     
     
	public void edit_selected_cd (CDItem cd)
	{
		appbar.Push (Catalog.GetString ("Updating CD"));
		db.update_cd (cd);
		cdview.search_cds (entry_search.Text.ToLower());
		appbar.Pop ();     
    }
     
	public void edit_selected_file (FileItem f,Pixbuf thumbnail)
	{
		// Update file in DB
		appbar.Push (Catalog.GetString ("Updating File ..."));
		db.update_file (f,thumbnail);
		
		// Update file in FilesView
		filesview.update_file (f);
	
		// Select this path
		filesview.select_path (f.Path);

		// Update details view
		lbl_info.Markup = f.Info;
	    	
    	// Try to read the thumbnail
   		img_thumb.Pixbuf = db.read_thumbnail (f.CD_Key,f.ThumbKey()); 

		// Done
		appbar.Pop ();
    }
 
 
 
     

	/* EVENTS */
    public void on_filesview_popup_click (string action)
	{
		ArrayList files_sel;
		
		switch (action.ToLower()) {
		
			case "edit":
				// Make sure there is only one selected files
				files_sel = filesview.get_selected_files();
				
				if (files_sel.Count!=1) {
					Console.WriteLine (Catalog.GetString ("ERROR: FilesView Popup menu selects 'edit' on more than one file"));
				} else {
					FileItem f = (FileItem)files_sel[0];
					edit_file_details (f); 
				}
				break;
			}
	}
	
	public void on_filesview_popup (string widget)
	{
		filesview_popup.Popup (filesview.get_selected_files(),0,0);
	}

	public void on_details_buttonpress_event (object o, ButtonPressEventArgs args)
	{
		if (args.Event!=null) {
			if (args.Event.Button==3) {
				filesview_popup.Popup (filesview.get_selected_files(),0,0);
			}
		}		
	}

	public void edit_file_details (FileItem f)
	{
		/* Read thumbnail */
		Pixbuf thumbnail = db.read_thumbnail (f.CD_Key,f.ThumbKey());		
		
		/* Create edit window */
		EditFileWindow win_editfile = new EditFileWindow (f,thumbnail,win_main);
				
		/* Associate handle with OK button */
		win_editfile.EditFileEvent += new EditFileWindow.EditFileEventHandler(edit_selected_file);
	}

	public void on_cdview_edit (int key)
	{
		on_edit_activate (null, null);
    }

	public void on_cdview_delete_selected (ArrayList cds_sel)
	{
		string msg;

		/* cds_sel is always >=1 */						
			
		/* Ask the user to delete CDs */
		if (cds_sel.Count==1) {
			msg = String.Format (Catalog.GetString ("Are you sure to delete CD named \"<b>{0}</b>\"?"),((CDItem)cds_sel[0]).Name);
		} else {
			msg = String.Format (Catalog.GetString ("Are you sure to delete the {0} selected CDs?"),cds_sel.Count.ToString());
		}	
		DeleteDialog dialog = new DeleteDialog (null, msg, win_main);	
		if (dialog.GetAnswer () == true) {

			/* Stop selection handler until we finish */
			cdview.stop_selection();

			/* Ok, delete the cds */
			appbar.Push (Catalog.GetString ("Deleting ..."));
			
			for (int i=0; i<cds_sel.Count; i++) {
				CDItem cd = (CDItem)cds_sel[i];
				db.delete_cd_and_files (cd.Key);
			}

			appbar.Pop ();
					
			/* Refresh the view */
			appbar.Push (Catalog.GetString ("Searching CDs"));
			cdview.search_cds ( entry_search.Text.ToLower() );
			appbar.Pop ();
				
			/* Activate selection handler again */
			cdview.resume_selection ();
	
			/* Fire a selection handler */
			ArrayList cds_sel_new = cdview.get_selected_cds ();
			on_cdview_selection_change (cds_sel_new);
   		}
   		  
    }    
	
	public void on_cdview_selection_change (ArrayList cds_sel)
    {
		switch (cds_sel.Count) {
			
			case 0: /* None is selected */
				appbar.ClearStack ();
				lbl_info.Markup = "";
				img_thumb.Pixbuf = null;
				filesview.Clear ();
				break;
				
			case 1: // One is selected, show files

				CDItem cd = (CDItem)cds_sel[0];
				if (filesview.cdkey != cd.Key) {
				 
					appbar.Push (Catalog.GetString ("Reading selected CD files ..."));

					// Delete previously selected CDs
					filesview.Clear ();
					lbl_info.Markup = "";
					img_thumb.Pixbuf = null;
				
					// Make sure the message is displayed
					for (int i=0; i<10; i++) {
						Gtk.Application.RunIteration (false);
					}

					// Get cd and files with selection
					Trace.WriteLine ("Query DB for files.");
					Hashtable files = db.read_files (cd.Key);

					// Fill treeview with files
					Trace.WriteLine ("Showing queried files.");
					filesview.fill (cd,files);

					appbar.Push (String.Format (Catalog.GetString ("Selected CD with {0} files"), files.Count));
					
				}
								
				break;
				
			default: // More than one is selected
				appbar.Push (String.Format (Catalog.GetString ("Selected {0} CDs"), cds_sel.Count));
				filesview.Clear ();
				lbl_info.Markup = "";
				img_thumb.Pixbuf = null;
				break;
		}
	}
        
 
	public void on_win_main_size_allocate (object o, Gtk.SizeAllocatedArgs args)
	{

		/* Save window position */
		int h,w;
        	
		win_main.GetSize(out w, out h);
        	
       	conf.setkey ("gui/main_height", h);
		conf.setkey ("gui/main_width", w);

	}
        
	public void on_hpanel_size_allocate (object o, Gtk.SizeAllocatedArgs args)
	{
		/* Write hpanel position */
        conf.setkey ("gui/main_hpanel_pos", hpanel.Position);
			
    }                
        
	public void on_vpanel_size_allocate (object o, Gtk.SizeAllocatedArgs args)
    {
		/* Write vpanel position */
        conf.setkey ("gui/main_vpanel_pos", vpanel.Position);
			
	}                
        
    public void on_win_main_delete_event (object o, DeleteEventArgs args) 
	{
        Quit ();
        args.RetVal = true;
	}

	public void on_menu_quit_activate (object o, EventArgs args) 
	{
		Quit ();
	}

	public void on_entry_search_changed (object o, EventArgs args)
	{
	
		/* Search CDs with entry name */
		appbar.Push(Catalog.GetString ("Searching CDs"));
		cdview.search_cds ( (o as Gtk.Entry).Text.ToLower() );
		appbar.Pop();
		
	}

	public void on_about_activate (object o, EventArgs args)
	{

		string [] authors = new String [] {
			"Javier Ruiz Hidalgo <jrhbcn@gmail.com>\n" +
            "Jacek Tomasiak <jacek.tomasiak@gmail.com>\n" + 
			"Michal Turek <michal.turek@gmail.com>"
		};

		string [] documenters = new String [] {
			"None"
		};

		string translators = 
			"Czech (cs): Ondra Kudlik <kepi@orthank.net>\n" +
			"Czech (cs): Michal Turek <michal.turek@gmail.com>\n" +
			"Spanish (es): Javier Ruiz Hidalgo <jrhbcn@gmail.com>";
	
		Pixbuf pixbuf = new Pixbuf(null, "cdcollect.png");

		Gtk.AboutDialog about = new Gtk.AboutDialog ();
		about.Name = "CDCollect";
		about.Version = conf.version;
		about.Website = "http://cdcollect.sourceforge.net";
		about.Comments = Catalog.GetString ("A GNOME CD/DVD catalog application");
		about.Authors = authors;
		about.Documenters = documenters;
		about.TranslatorCredits = translators;
		about.Logo = pixbuf;
	
		about.Run ();

	}


	public void on_find_activate (object o, EventArgs args)
	{        
		/* Create find collection object */
		Trace.WriteLine ("Creating FindCollectionWindow.");
		win_find = new FindCollectionWindow (win_main);

        /* Associate events */
        win_find.FileSelectedEvent += find_file_selected;	   
   	}
	
	private void find_file_selected (int key, string p)
	{
		/* Clear view on cds */
		entry_search.Text = "";
		
		/* Select CD on CDView */
		cdview.Selection.UnselectAll ();
		cdview.select_cd(key);
		
		/* Select current path */
		filesview.CollapseAll ();
		filesview.select_path (p);
	}

	public void on_import_activate (object o, EventArgs args)
	{
		/* Create import window */
		ImportCDWindow win_importcd = new ImportCDWindow(
			(string) conf.getkey ("prefs/mount_point",conf.default_cd_mount),
			(bool)   conf.getkey ("prefs/use_information_plugins",true),
			(bool)   conf.getkey ("prefs/use_compression_plugins",true),
			(bool)   conf.getkey ("prefs/compression_plugins_recursive",true),
			(bool)   conf.getkey ("prefs/generate_thumbnails",true),
			win_main
		);

		/* Associate handles with OK button */
		win_importcd.ImportCDEvent += new ImportCDWindow.ImportCDEventHandler(on_importcd_ok_activate);
		
	}

	//public void on_importcd_ok_activate (string mount, string name, string keywords, DateTime date, string icon)
	public void on_importcd_ok_activate (int key)
	{
		/* Check if CD has been read */
		if (key==-1) {
			new ErrorDialog (String.Format (Catalog.GetString (
				"Directory <b>{0}</b> not found.\nPlease change the CD mount point in the Preferences window."), 
				(string) conf.getkey ("prefs/mount_point",conf.default_cd_mount)), win_main);
		} else if (key==-2) {
			new ErrorDialog (Catalog.GetString ("The mount directory is empty. Please make sure the CD is mounted."), win_main);	
		} else if (key==-3) {
			new ErrorDialog (Catalog.GetString ("Unknown error when importing CD. Please fill a bug report."), win_main);	
		} else {

		   /* Create new item, Add it to the database */
		   //appbar.Push (Catalog.GetString ("Importing CDs"));
		   //db.import_cd (mount, name, keywords, date, icon, true);
		   //appbar.Pop ();		

		   /* Re-do the search */
		   appbar.Push (Catalog.GetString ("Searching CDs"));
		   cdview.search_cds (entry_search.Text.ToLower());
		   appbar.Pop ();
	   
		   /* Select it if possible */
		   cdview.select_cd (key);

		}
	}

	private void on_edit_activate (object o, EventArgs args)
	{
		/* Create edit window */
		ArrayList cds = cdview.get_selected_cds();

		if (cds.Count==1) {		
			EditCDWindow win_editcd = new EditCDWindow ((CDItem)cds[0], win_main);
				
			/* Associate handle with OK button */
			win_editcd.EditCDEvent += new EditCDWindow.EditCDEventHandler(on_editcd_ok_activate);
		}		
	}
	
	public void on_editcd_ok_activate (CDItem cd)
	{        
		edit_selected_cd (cd);
	}

	public void on_prefs_activate (object o, EventArgs args)
	{
		/* Create Preferences window */
		new PreferencesWindow(
			(string) conf.getkey ("prefs/mount_point",conf.default_cd_mount),
			(bool)   conf.getkey ("prefs/use_information_plugins",true),
			(bool)   conf.getkey ("prefs/use_compression_plugins",true),
			(bool)   conf.getkey ("prefs/compression_plugins_recursive",true),
			(bool)   conf.getkey ("prefs/generate_thumbnails",true),
			win_main
		 );
			
	}

	public void on_plugins_activate (object o, EventArgs args)
	{
		/* Create Plugins window */
		new PluginsWindow(win_main);
			
	}

	public void on_filesview_selection_change (object o, EventArgs args) 
	{
		
		TreeModel model;
		TreeIter iter;
		
		if ( ((TreeSelection)o).GetSelected (out model, out iter) ) {
			
			/* Display info on the label */			
			string info = (string) model.GetValue(iter, filesview.column_info);
			lbl_info.Markup = info;
	    	
	    	/* Try to read the thumbnail */
	    	string thumbkey  = (string) model.GetValue(iter, filesview.column_thumbkey);
    		img_thumb.Pixbuf = db.read_thumbnail (filesview.cdkey,thumbkey); 
	    	
	    	/* Mark selected row on the cd item */
	    	if ( db.cds.Contains(filesview.cdkey) ) {
				((CDItem) db.cds[filesview.cdkey]).Selected = model.GetPath(iter).ToString();
			}

		}

	}

}

