/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Xml;

using Mono.Unix;

using Gtk;
using Gdk;
using Glade;
using GConf;
using Gnome; 


public class PluginsWindow : Gtk.Window
{
             
	/* Glade Widgets */
	[Glade.Widget] public Gtk.Window win_plugins;
	
	[Glade.Widget] public Gtk.Button btn_close;
	
	[Glade.Widget] public Gtk.TreeView tv_info;
	[Glade.Widget] public Gtk.TreeView tv_comp;

	[Glade.Widget] public Gtk.Button btn_info_add;
	[Glade.Widget] public Gtk.Button btn_info_edit;
	[Glade.Widget] public Gtk.Button btn_info_remove;

	[Glade.Widget] public Gtk.Button btn_comp_add;
	[Glade.Widget] public Gtk.Button btn_comp_edit;
	[Glade.Widget] public Gtk.Button btn_comp_remove;


	/* Public variables */
        
	/* Private variables */
	private static Glade.XML gxml;
	
    /* Methods */        
	public PluginsWindow (Gtk.Window parent) : base (IntPtr.Zero) 
	{
		/* Show Main Dialog using glade */
 		gxml = new Glade.XML (null, "gui.glade", "win_plugins", null);
			
		/* Connect functions to events */
		gxml.Autoconnect (this);
		
		win_plugins.TransientFor = parent;

		/* Prepare gui */
		prepare_gui ();

		/* Create List Stores */
		tv_info.Model = new ListStore (typeof(string),typeof(string));
		tv_comp.Model = new ListStore (typeof(string),typeof(string));
		
        /* Append columns in the treeview */
        TreeViewColumn tc;
  		tc = tv_info.AppendColumn (Catalog.GetString ("Mime"), new CellRendererText(), "text", 0); tc.Resizable = true;
        tc = tv_info.AppendColumn (Catalog.GetString ("Program"), new CellRendererText(), "text", 1); tc.Resizable = true;
        				
        tc = tv_comp.AppendColumn (Catalog.GetString ("Mime"), new CellRendererText(), "text", 0); tc.Resizable = true;
        tc = tv_comp.AppendColumn (Catalog.GetString ("Program"), new CellRendererText(), "text", 1); tc.Resizable = true;

		
		/* Set Sorting in tree views */
     	(tv_info.Model as ListStore).SetSortColumnId( 0, Gtk.SortType.Ascending);
     	(tv_comp.Model as ListStore).SetSortColumnId( 0, Gtk.SortType.Ascending);

		
		/* Associate events in treeviews */
		tv_info.Selection.Changed += new EventHandler (on_info_selection_change);
		tv_comp.Selection.Changed += new EventHandler (on_comp_selection_change);

		tv_info.RowActivated += new RowActivatedHandler (on_info_row_activate);		
		tv_comp.RowActivated += new RowActivatedHandler (on_comp_row_activate);		


		/* Set info and comp plugins */	
		fill_info();
		fill_comp();
		
		btn_info_edit.Sensitive = false;
		btn_info_remove.Sensitive = false;
		btn_comp_edit.Sensitive = false;
		btn_comp_remove.Sensitive = false;


		/* Show all widgets */
		win_plugins.Modal = true;
		win_plugins.ShowAll ();
           				        
	}
	
	private void prepare_gui()
	{
      	
		/* Title */
		win_plugins.Icon = new Pixbuf(null, "cdcollect_small.png");
		win_plugins.Title = Catalog.GetString ("CDCollect: Plugins");

		/* Read GUI gconf properties */
		int win_height, win_width;
      		 
		/* Window size */
		win_height = (int) CDCollect.conf.getkey ("gui/plugins_height", 300);
		win_width  = (int) CDCollect.conf.getkey ("gui/plugins_width", 200);
		win_plugins.Resize( win_width, win_height);			
				
	}

	public void on_size_allocate (object o, Gtk.SizeAllocatedArgs args)
	{

		/* Save window position */
		int h,w;
        	
		win_plugins.GetSize(out w, out h);
        	
       	CDCollect.conf.setkey ("gui/plugins_height", h);
		CDCollect.conf.setkey ("gui/plugins_width", w);

	}

	private void append (InfoPluginItem pitem)
	{
		(tv_info.Model as ListStore).AppendValues (pitem.Mime, pitem.Program);
	}
	private void append (CompPluginItem c)
	{
		(tv_comp.Model as ListStore).AppendValues (c.Mime, c.Program);
	}


	private void fill_info ()
	{	
		foreach (InfoPluginItem pitem in CDCollect.db.infoplugins.Values) {
			append (pitem);
		}
	}
	private void fill_comp ()
	{	
		foreach (CompPluginItem c in CDCollect.db.compplugins.Values) {
			append (c);
		}
	}

	public void on_close_clicked (object o, EventArgs args)
	{
		win_plugins.Destroy();
	}

    public void on_delete_event (object o, DeleteEventArgs args) 
	{
		win_plugins.Destroy();
	}

	public void on_info_selection_change (object o, EventArgs args) 
	{
		TreeModel model;
		TreeIter iter;
		
		if ( ((TreeSelection)o).GetSelected (out model, out iter) ) {
			//string mime = (string) model.GetValue(iter, 0);
			btn_info_edit.Sensitive = true;
			btn_info_remove.Sensitive = true;
		} else {
			btn_info_edit.Sensitive = false;
			btn_info_remove.Sensitive = false;
		}

	}
	public void on_comp_selection_change (object o, EventArgs args) 
	{
		TreeModel model;
		TreeIter iter;
		
		if ( ((TreeSelection)o).GetSelected (out model, out iter) ) {
			//string mime = (string) model.GetValue(iter, 0);
			btn_comp_edit.Sensitive = true;
			btn_comp_remove.Sensitive = true;			
		} else {
			btn_comp_edit.Sensitive = false;
			btn_comp_remove.Sensitive = false;		
		}

	}
	
	public void on_btn_info_remove_clicked (object o, EventArgs args)
	{
		TreeModel model;
		TreeIter iter;
				
		if ( tv_info.Selection.GetSelected (out model, out iter) ) {

			/* Get mime type to delete */
			string mime = (string) model.GetValue(iter, 0);

			/* Ask the user to delete infoplugin */
			string msg = String.Format (Catalog.GetString ("Are you sure to delete information plugin on mime type \"<b>{0}</b>\"?"),mime);
			DeleteDialog dialog = new DeleteDialog (null, msg, win_plugins);
			if (dialog.GetAnswer () == true) {
				CDCollect.db.delete_infoplugin (mime);
				/* Re-fill the treeview */
				(tv_info.Model as ListStore).Clear();
				fill_info();
			}	
		}
		
	}
	public void on_btn_comp_remove_clicked (object o, EventArgs args)
	{
		TreeModel model;
		TreeIter iter;
				
		if ( tv_comp.Selection.GetSelected (out model, out iter) ) {

			/* Get mime type to delete */
			string mime = (string) model.GetValue(iter, 0);

			/* Ask the user to delete infoplugin */
			string msg = String.Format (Catalog.GetString ("Are you sure to delete compression plugin on mime type \"<b>{0}</b>\"?"),mime);
			DeleteDialog dialog = new DeleteDialog (null, msg, win_plugins);
			if (dialog.GetAnswer () == true) {
				CDCollect.db.delete_compplugin (mime);
				/* Re-fill the treeview */
				(tv_comp.Model as ListStore).Clear();
				fill_comp();
			}	
		}
		
	}
	
	public void on_btn_info_edit_clicked (object o, EventArgs args)
	{
		TreeModel model;
		TreeIter iter;
				
		if ( tv_info.Selection.GetSelected (out model, out iter) ) {

			string mime = (string) model.GetValue(iter, 0);
			InfoPluginItem pitem = (InfoPluginItem) CDCollect.db.infoplugins[mime]; 
			
			/* Edit window */
			InfoPluginWindow win = new InfoPluginWindow (pitem, win_plugins);
			
			/* Add event handler */
			win.AddEvent += info_plugin_add;

		}
	
	}
	public void on_btn_comp_edit_clicked (object o, EventArgs args)
	{
		TreeModel model;
		TreeIter iter;
				
		if ( tv_comp.Selection.GetSelected (out model, out iter) ) {

			string mime = (string) model.GetValue(iter, 0);
			CompPluginItem c = (CompPluginItem) CDCollect.db.compplugins[mime]; 
			
			/* Edit window */
			CompPluginWindow win = new CompPluginWindow (c, win_plugins);
			
			/* Add event handler */
			win.AddEvent += comp_plugin_add;

		}
	
	}
	

	public void on_btn_info_add_clicked (object o, EventArgs args)
	{

		/* Add window */
		InfoPluginWindow win = new InfoPluginWindow (win_plugins);
			
		/* Add event handler */
		win.AddEvent += info_plugin_add;
	
	}
	public void on_btn_comp_add_clicked (object o, EventArgs args)
	{

		/* Add window */
		CompPluginWindow win = new CompPluginWindow (win_plugins);
			
		/* Add event handler */
		win.AddEvent += comp_plugin_add;
	
	}
	
	private void info_plugin_add (InfoPluginItem p, string oldmime)
	{
		if (oldmime=="") {
			/* Check if already exists */
			if (CDCollect.db.infoplugins.Contains(p.Mime)) {
				new ErrorDialog (
					String.Format (
						Catalog.GetString ("Information Plugin on mime '{0}' already exists, please use the edit button."),p.Mime
					)
				);
			} else {
				CDCollect.db.import_infoplugin (p);	

				/* Re-fill the treeview */
				(tv_info.Model as ListStore).Clear();
				fill_info();
			}
		} else {
			/* Delete old mime type if different */
			if (oldmime!=p.Mime) {
				CDCollect.db.delete_infoplugin (oldmime);
			}
			CDCollect.db.import_infoplugin (p);						

			/* Re-fill the treeview */
			(tv_info.Model as ListStore).Clear();
			fill_info();
		}
	}
	private void comp_plugin_add (CompPluginItem c, string oldmime)
	{
		if (oldmime=="") {
			/* Check if already exists */
			if (CDCollect.db.compplugins.Contains(c.Mime)) {
				new ErrorDialog (
					String.Format (
						Catalog.GetString ("Compression Plugin on mime '{0}' already exists, please use the edit button."),c.Mime
					)
				);
			} else {
				CDCollect.db.import_compplugin (c);	
        		/* Re-fill the treeview */
            	(tv_comp.Model as ListStore).Clear();
        		fill_comp();
			}
		} else {
			/* Delete old mime type if different */
			if (oldmime!=c.Mime) {
				CDCollect.db.delete_compplugin (oldmime);
			}
			CDCollect.db.import_compplugin (c);						
    		/* Re-fill the treeview */
        	(tv_comp.Model as ListStore).Clear();
    		fill_comp();
		}
		

	}
	
	public void on_info_row_activate (object o, RowActivatedArgs args)
	{
		TreeIter iter;
		string mime;

    	/* Get selected plugin */
		tv_info.Model.GetIter (out iter, args.Path);	
		mime = (string) tv_info.Model.GetValue(iter, 0);
		InfoPluginItem i = (InfoPluginItem) CDCollect.db.infoplugins[mime]; 
			
		/* Edit window */
		InfoPluginWindow win = new InfoPluginWindow (i, win_plugins);
			
		/* Add event handler */
		win.AddEvent += info_plugin_add;						
	}
	public void on_comp_row_activate (object o, RowActivatedArgs args)
	{
		TreeIter iter;
		string mime;

    	/* Get selected plugin */
		tv_comp.Model.GetIter (out iter, args.Path);	
		mime = (string) tv_comp.Model.GetValue(iter, 0);
		CompPluginItem c = (CompPluginItem) CDCollect.db.compplugins[mime]; 
			
		/* Edit window */
		CompPluginWindow win = new CompPluginWindow (c, win_plugins);
			
		/* Add event handler */
		win.AddEvent += comp_plugin_add;		
				
	}
		
	

}

