/*
 * Copyright (C) 2004 Javier Ruiz Hidalgo <jrhbcn@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Xml;

using Mono.Unix;

using Gtk;
using Gdk;
using Glade;
using GConf;
using Gnome; 


public class PreferencesWindow : Gtk.Window
{
             
	// Glade Widgets
	[Glade.Widget] public Gtk.Window win_prefs;
	[Glade.Widget] public Gtk.Button btn_ok;
	[Glade.Widget] public Gtk.Button btn_cancel;
	[Glade.Widget] public Gtk.Entry  entry_mount;
    
    [Glade.Widget] public Gtk.CheckButton cbtn_info;
	[Glade.Widget] public Gtk.CheckButton cbtn_comp;
	[Glade.Widget] public Gtk.CheckButton cbtn_recursive;
	[Glade.Widget] public Gtk.CheckButton cbtn_thumbnail;
    
	// Public variables
        
	// Private variables
	private static Glade.XML gxml;
	
    // Methods        
	public PreferencesWindow (string mount, bool info, bool comp, bool recursive, bool thumbs, Gtk.Window parent) : base (IntPtr.Zero) 
	{
		// Show Main Dialog using glade
 		gxml = new Glade.XML (null, "gui.glade", "win_prefs", null);
			
		// Connect functions to events
		gxml.Autoconnect (this);
		
		win_prefs.TransientFor = parent;

		// Title
		win_prefs.Icon = new Pixbuf(null, "cdcollect_small.png");
		win_prefs.Title = Catalog.GetString ("CDCollect: Preferences");

		// Fill preferences
		entry_mount.Text = mount;
		cbtn_info.Active = info;
		cbtn_comp.Active = comp;
		cbtn_recursive.Active = recursive;
		cbtn_thumbnail.Active = thumbs;
		
		// Check sensitive buttons
		on_cbtn_comp_toggled (null,null);
					
		// Show all widgets
		win_prefs.Modal = true;
		win_prefs.ShowAll ();
           				
		// Select focus          											    																		    											
        entry_mount.GrabFocus();
        
	}

	public void on_cbtn_comp_toggled (object o, EventArgs args)
	{
		if (cbtn_comp.Active) {
			cbtn_recursive.Sensitive = true;
		} else {
			cbtn_recursive.Sensitive = false;
		}		
	}

	public void on_ok_clicked (object o, EventArgs args)
	{
		CDCollect.conf.setkey ("prefs/mount_point", entry_mount.Text);
		CDCollect.conf.setkey ("prefs/use_information_plugins", cbtn_info.Active);
		CDCollect.conf.setkey ("prefs/use_compression_plugins", cbtn_comp.Active);
		CDCollect.conf.setkey ("prefs/compression_plugins_recursive", cbtn_recursive.Active);
		CDCollect.conf.setkey ("prefs/generate_thumbnails", cbtn_thumbnail.Active);
		win_prefs.Destroy();		
	}

	public void on_cancel_clicked (object o, EventArgs args)
	{
		win_prefs.Destroy();
	}

    public void on_delete_event (object o, DeleteEventArgs args) 
	{
		win_prefs.Destroy();
	}


}

